**************************************************************************c
c                                                                         c
c  Program: GFS_traject_3d_v01.f                                          c
c  Version of trajectory code that reads 3-d press-level fields             c
c  from the GEOS4 coordinate model, and computes 3-d trajectories           c
c  based on horizontal wind and eta-dot fields.                             c
c                                             tdf 07/12/2K5                 c
c                                                                           c
c  based on SOLVE_traject_3d_v18.f                                          c
c  reads UKMO,NMC or ECMWF assimilated isentropic level data and computes   c
c  isentropic trajectories using a fourth-order Runge-Kutta time step       c
c                                                                           c
c  Input data:           assimilated u,v,p,pv,q                             c
c  Input parameter file: SOLVE_traject_3d_v13.fil                           c
c  Output data:          trajectory x,y,z                                   c
c                                                                           c
c  Programed by: R. Bradley Pierce 24-aug-1993                              c
c                MS401B TSB                                                 c
c                NASA LARC                                                  c
c                                                                           c
c  Version 2 updates:                                                       c
c                                                                           c
c  Programed by T. Duncan Fairlie 3-nov-1993                                c
c               MS401B TSB                                                  c
c               STC                                                         c
c                                                                           c
c  replaced 24 (hours) by dtflow in expressions for ystep and nstep         c
c  removed nc, nr and x, y from output file header separate subroutines     c
c  for writing and reading header information changed unit number for       c
c  restart file from 10 to 13 to avoid confusion with flow field files      c
c  removed variables day1 and dayn (not used anywhere)                      c
c  removed variable DAY from main program (incremented but not used)        c
c  introduced subroutine calendar to provide current calendar time          c
c  added integer*4 variables istime, ictime of the form "yymmddhh"          c
c  to parameter block comm_ashoe_3d_v08. istime is read in as input         c
c  used dtout as criterion for writing out output file. wtime is the        c
c  accumulated time between writes. subroutine write_output created.        c
c  nrday replaced with irtime - restart time in "yymmddhh" format           c
c                                                                           c
c  additional updates: changed stop criteria for streamline initialization  c
c                      to avoid overshooting initial longitude              c
c                      Pierce 12/7/93                                       c
c                                                                           c
c                      corrected overwrite of xp,yp in interp which caused  c
c                      <10km 'pearl necklace' problems as parcels passed    c
c                      1dx from 0 degrees (1/24/93)                         c
c                                                                           c 
c ***  fix wtime=-12. initially so that second dump is 36 hours into run
c      with 24 hours therafter
c
c  Version 4 updates:
c
c  test: for back trajectory with 12Z winds initialising at 00Z -
c        set ictime = 00Z time
c        set etime = 12 and wtime = 12 initially
c        call initialisation when time becomes 0.0
c        initialisation subroutine introduced
c
c  Version 6 updates:
c        additional option of initialising parcels inside the ER2
c        range ring: changes to comm_ashoe_v06 and header information
c
c  Version 7 updates:
c        inclusion of daylit and night-time fractional exposure
c        for each parcel: frday(nmax),frnght(nmax)  between outputs.
c        julday and gmt, and solar zenith angle calculations added.
c 						Duncan 4/10/97
c  Version 8 updates:
c        reads ecmwf data
c        3 dimensional
c        new calendar subroutine 		VLH 1/28/98
c
c  Version 9 updates:
c        reads UKMO netCDF isentropic data	VLH 9/22/98
c        if streamline initialisation then reads UKMO nondivergent winds
c        needs comm_uwhyb_3d_v09. to declare xmsf arrays
c
c  Version 10 updates:
c        changed er2leg to flight_path          VLH 10/14/98
c        capable of initializing an ER2 flight track or DIAL curtain
c        requires v10 of .fil to read in DIAL parameters
c        v09 comm file contains v10 DIAL parameters
c
c  Version 11 updates:
c        introduce time-dependent initialisation of DIAL data
c        requires v11 of .fil to read in DIAL parameters
c        v11 comm file contains v11 DIAL parameters
c                                              Duncan 6/28/99 
c  Version 12 updates:
c        introduce velocity potential and stream function
c        2 marker fields read, xmrkpv and xmrksf    
c                                              Duncan 10/13/99
c                                                                           c
c  'etime' is the elapsed time of the model run                             c
c  Version 13 updates:
c        introduced t0n and agen variables
c          (initial time and age) for DIAL applications
c                                              Duncan 11/05/99
c  Version 15 updates:
c        introduced minimum temperature variable xmint 
c                                              Duncan 7/18/2000
c
c  Version 16 updates:
c        introduce 3 .nc4 marker fields
c        xmrkpvl, xmrksfl, xmrksfh
c                                              VLH 8/24/2001
c
c  Retain discrete nature of the vortex marker field.  Do not interpolate
c  in time or space.  In time, change value at 12Z.  In space, assign the
c  majority of vertex values.  In case of a tie set to zero.
c                                              VLH 5/1/2002
c
c  Version 17 updates:
c        include option to initialize annulus around dc8 flight track
c    altitude
c                                              VLH 9/10/2002
c
c  Version 18 updates:
c        read only 1 marker field (xmrksf) that is equivalent to xmrksfl and 
c    xmrksfh. polar vortices are marked 1 and anticyclones are negative integers
c    remove streamfunction and velocity potential fields
c                                              VLH 12/4/2002
c  Version 20 updates
c     New eta coordinate version, reading hybrid data from UWhybrid model
c     Change in variable meanings:
c     (1) dyanmical variables: 
c         WVEL variables now refer to pdot. (formerly refered to theta-dot) 
c     (2) Trajectory variables: 
c        "q..." names eliminated, replaced by "thd..." (theta-dot). 
c     New added variables: thn, shn
c                                              tdf 11/05/2K4
c     Introduced new rdwind routine that reads in NetCDF variables by name
c     rather than by order. This eliminates arr1, arr2, arr3, etc.
c                                              tdf 01/05/2K5
c     Introduced option to read Henry Fuelberg initial parcel positions 
c                                              tdf 02/04/2K5
c     Introduced option to read 1-minute curtain data, derived from RAQMS
c     1-second curtain files (see "make_1-minute_curtain_data.pro")
c                                              tdf 04/29/2K5
c  GFS version updates
c  changed getdiag
c          interp_gfs
c  Introduced CALIPSO initialisation           tdf 8/19/2K5
c  which mimics DIAL initialisation
c          read_calipso
c          init_calipso
c          
c***************************************************************************c
      program GFS_traject_3d_v01
c
      include "netcdf.inc"
c
      include 'comm_gfs_3d_v01.'
c
      REAL UVELOLD(NRmax,NCmax,nthwmx) 
      REAL VVELOLD(NRmax,NCmax,nthwmx) 
ctdf Geop. altitude
      REAL ALTOLD(NRmax,NCmax,nthwmx) 
c p-dot   (mbar/s)
      REAL WVELOLD(NRmax,NCmax,nthwmx) 
      REAL POLD(NRmax,NCmax,nthwmx) 
      REAL PVOLD(NRmax,NCmax,nthwmx) 
      REAL XMSFOLD(NRmax,NCmax,nthwmx)
      REAL QDFOLD(NRmax,NCmax,nthwmx) 
ctdf Introduce multiple new arrays    tdf 10/29/2K4
      REAL SHOLD(NRmax,NCmax,nthwmx)
      REAL THOLD(NRmax,NCmax,nthwmx)
ctdf      REAL THDOLD(NRmax,NCmax,nthwmx)
      REAL PSFCOLD(NRmax,NCmax)
C
      REAL UVELNEW(NRmax,NCmax,nthwmx) 
      REAL VVELNEW(NRmax,NCmax,nthwmx)
ctdf Geop. altitude
      REAL ALTNEW(NRmax,NCmax,nthwmx)
c p-dot   (mbar/s)
      REAL WVELNEW(NRmax,NCmax,nthwmx)
      REAL PNEW(NRmax,NCmax,nthwmx) 
      REAL PVNEW(NRmax,NCmax,nthwmx)
      REAL XMSFNEW(NRmax,NCmax,nthwmx)
      REAL QDFNEW(NRmax,NCmax,nthwmx)
ctdf Introduce multiple new arrays    tdf 10/29/2K4
      REAL SHNEW(NRmax,NCmax,nthwmx)
      REAL THNEW(NRmax,NCmax,nthwmx)
ctdf      REAL THDNEW(NRmax,NCmax,nthwmx)
ctdf surface pressure                          tdf 7/29/2K5
      REAL PSFCNEW(NRmax,NCmax)
c
c Eliminated arrays arr1, arr2, arr3, etc.      tdf 01/05/2K5
c
      logical ex0  
c
      character*120 savfile
      character*120 test
      character name*20
      integer*4 date, idim
c
      data ex0 /.false./
c
c  read input parameters for this experiment
c
      print *, 'Opening GFS_traject_3d_v01.fil'
      OPEN(5,FILE='GFS_traject_3d_v01.fil',status='old',READONLY)
      call read_list_header(5)
c
c  write input parameters for this experiment to day file
c
      call write_list_header(6)
c
c  truncate blank characters from the name of the input/output file directory
c
      len=0
      len1=0
      test=ifile(1)
      do n=1,119 
        if(dir(n:n+1).ne.' ')len=len+1
        if(test(n:n+1).ne.' ')len1=len1+1
      enddo
C
      OPEN(12,FILE=OFILE,status='new',FORM='UNFORMATTED')
c
c  write input parameters for this experiment to output file
c
      call write_unf_header(12)
c
c  test for restart from existing trajectory data set
c
      if(restart) then
        OPEN(13,FILE=dir(1:len)//rFILE,status='old',
     &       FORM='UNFORMATTED')
        call read_restart_data(13)
        close(13)
      endif
c
      if ( .not. restart) then
        call zero_arrays
      endif
c
c initialize dependent time loop variables
c
      ystep=abs(60.*dtflow/dt)
      idt=dt
      idtflw=dtflow
      nstep=abs(60*idtflw/idt)
c      print *,'ystep= ',ystep
c      print *,'nstep= ',nstep
c
c initialize constants
c
      dt=dt*60.
      PI2 = 6.2831853071796
      print *,'dt=',dt, 'seconds'
      RAD=6.37E6
      COUNT=0
      ITFLAG=0 
C 
c  next two lines for back trajectory with 0Z winds & output at 0Z
C      if (.not. restart) etime=0.0    
C      wtime=0.0
      if (.not. restart) etime=0.0 
c
c UKMO valid at 12 Z
c
c      wtime=12.0
      wtime=0.0
c     wtime=-12.0  ! fix so that second dump is 36 hours into run and 24 after
c
c  read flow field data from unit 10
c
      if (.not. restart) then
        inquire(file=dir(1:len)//test(1:len1),exist=ex0)
c
c open netCDF dataset w/o write privileges
        if (ex0) then
          ncid = ncopn(dir(1:len)//test(1:len1),
     &                 NCNOWRIT, iret)
          print *,'opening ',dir(1:len)//test(1:len1)
        endif
        if (.not. ex0) then
          print *,' error in locating flow file: ex0 =',ex0
          stop
        endif 
c
c get dimensions
        idim=ncdid(ncid,'number_of_longitudes',         iret)
        call ncdinq(ncid, idim, name, nc,  iret)
        idim=ncdid(ncid,'number_of_latitudes',         iret)
        call ncdinq(ncid, idim, name, nr,  iret)
        idim=ncdid(ncid,'number_of_levels',         iret)
        call ncdinq(ncid, idim, name, nth, iret)
ctdf Now nth is the number of eta levels        tdf 10/26/2K4
        write(6,*)'netCDF dimensions ',nr,nc,nth
        ifirst=0
        ncw=nc
        nrw=nr
        nthw=nth
c
c new version of rdwind, rdwind_name, that reads met variables  
c by name rather than by order. This eliminates the need for   
c                                            tdf 01/05/2K5
        call rdwind(ncid,ncw,nrw,nthw,ifirst)
      endif
      if (restart) then
        inquire(file=dir(1:len)//test(1:len1),exist=ex0)
        if (ex0) then
          ncid = ncopn(dir(1:len)//test(1:len1),
     &                 NCNOWRIT, iret)
        endif
        if (.not. ex0) then
          print *,' error in locating flow file: ex0 =',ex0
          stop
        endif 
c
c get dimensions
        idim=ncdid(ncid,'latitudes',         iret)
        call ncdinq(ncid, idim, name, nr,  iret)
        idim=ncdid(ncid,'longitudes',         iret)
        call ncdinq(ncid, idim, name, nc,  iret)
        idim=ncdid(ncid,'eta_levels',         iret)
        call ncdinq(ncid, idim, name, nth, iret)
ctdf Now nth is the number of eta levels        tdf 10/26/2K4
        write(6,*)'netCDF dimensions ',nr,nc,nth
        ifirst=0
        ncw=nc
        nrw=nr
        nthw=nth
c
c new version of rdwind, rdwind_name, that reads met variables  
c by name rather than by order. This eliminates the need for   
c                                            tdf 01/05/2K5
        call rdwind(ncid,ncw,nrw,nthw,ifirst)

      endif
c
      wmax=0.
      xlatmax=0
      DO 12 L=1,nth
      DO 12 I=1,NC
        DO 12 J=1,NR
c tdf NB Changed ordering of "sim" arrays        tdf 02/04/2K5
           ugrd(j,i,l)=usim(i,j,l)
           vgrd(j,i,l)=vsim(i,j,l)
c geop. altitude                                 tdf 09/09/2K5
           altgrd(j,i,l)=altsim(i,j,l)
           xmsfgrd(j,i,l)=xmsfsim(i,j,l)
           pgrd(j,i,l)=psim(i,j,l)
           pvgrd(j,i,l)=pvsim(i,j,l)
ctdf wgrd now contains p-dot (mbar/s)            tdf 10/29/2K4
           wgrd(j,i,l)=wsim(i,j,l)
           qdfgrd(j,i,l)=qdfsim(i,j,l)
ctdf Add in spec. hum. and theta, and theta-dot
           shgrd(j,i,l)=shsim(i,j,l)
           thgrd(j,i,l)=thsim(i,j,l)
ctdf           thqdfgrd(j,i,l)=thdsim(i,j,l)
12    CONTINUE
      wmax=sqrt(wmax)
ctdf surface pressure                          tdf 7/29/2K5
      DO 13 I=1,NC
        DO 13 J=1,NR
           psfcgrd(j,i)=psfcsim(i,j)
13    CONTINUE
 
c  for a restart run, no new trajectory initialisation is needed
      if (restart) goto 25
 
c  starting with clean sheets: initialise trajectories;
c  compute initial gmt and julian day             Duncan 4/10/97
c  when etime becomes 0.0
c
      if ( abs(etime) .lt. 0.1 ) then
c  comment out when initializing at 12Z with 0Z winds
        call initial
        call jul_gmt (istime,julday,gmt)
        if (dial) then
c output initial characteristics of DIAL parcels 
          if (dt .gt. 0.0) then           ! forward trajectories 
            do id = 1, ncount
              idial = id  
ctdf              call init_dial(idial)
              call init_dial
              write(6,340) idial, ntraj 
            end do
          endif
          if (dt .lt. 0.0) then           ! back trajectories 
            do id = ncount,1,-1
              idial = id 
ctdf              call init_dial(idial)
              call init_dial
              write(6,340) idial, ntraj 
            end do
          endif 
        endif 
ctdf Add CALIPSO option                    tdf 8/22/2K5
        if (calipso) then
c output initial characteristics of CALIPSO parcels 
          if (dt .gt. 0.0) then           ! forward trajectories 
            do id = 1, ncount
              idial = id  
              call init_calipso(ist,last)
              write(6,340) idial, ntraj 
            end do
          endif
          if (dt .lt. 0.0) then           ! back trajectories 
            do id = ncount,1,-1
              idial = id 
              call init_calipso(ist,last)
              write(6,340) idial, ntraj 
            end do
          endif 
        endif 
ctdf Add curtain option                    tdf 4/18/2K5
        if (curtain) then
c output initial characteristics of curtain parcels
          if (dt .gt. 0.0) then           ! forward trajectories 
            do id = 1, ncount
              idial = id  
              call init_curtain
              write(6,340) idial, ntraj 
            end do
          endif
          if (dt .lt. 0.0) then           ! back trajectories 
            do id = ncount,1,-1
              idial = id 
              call init_curtain
              write(6,340) idial, ntraj 
            end do
          endif 
        endif 

      endif
  340 format (' # ',I5,' start time                  ',
     1         '                    ntraj =',I6)
c
25    continue
c
C  WRITE OUTPUT
C
      if (ntraj .lt. 1) then
c adjust etime up for call to calendar         Duncan 5/12/2000 
        rtime = etime + 0.0001
        call calendar(istime,rtime,ictime)
        print *,' No trajectories found at output time',ictime 
      else
c  The initial DIAL data includes dynamic characteristics  Dunc 7/21/99
c  The initial DC-8 data includes dynamic characteristics  VLH 9/10/2002
c  The initial curtain data includes dynamic characteristics tdf 4/29/2K5

c  The initial CALIPSO data lacks dynamic characteristics  tdf 8/22/2K5
        if (.not. dial .and. .not. dc8 .and. .not. curtain
     1  .and. .not. calipso) call getdiag
c
c adjust etime up for call to calendar         Duncan 5/12/2000 
        rtime = etime + 0.0001
        call calendar(istime,rtime,ictime)
        write(6,*)'writing output at time',ictime,' ntraj =',ntraj
        call write_output(12)
      endif
c
c re-zero dial or DC-8 parcels 
      if (dial .or. dc8 .or. curtain .or. calipso) then
        ntraj = 0
        if (dt .gt. 0.0) idial = 1
        if (dt .lt. 0.0) idial = ncount 
      endif 
c
c  looping over flow field files, compute trajectories
c
      DO 999 NFILE=1,NFILES
        test=ifile(nfile)
c  Error corrected: len1 reset                            Duncan 12/04/99 
        len1=0
        do n=1,119
          if(test(n:n+1).ne.' ')len1=len1+1
        enddo
C
        inquire(file=dir(1:len)//test(1:len1),exist=ex0)
c
c open netCDF dataset w/o write privileges
        if (ex0) then
          ncid = ncopn(dir(1:len)//test(1:len1),
     &                 NCNOWRIT, iret)
          print *,'opening ',dir(1:len)//test(1:len1)
        endif
c
c get dimensions
        idim=ncdid(ncid,'number_of_longitudes',         iret)
        call ncdinq(ncid, idim, name, nc,  iret)
        idim=ncdid(ncid,'number_of_latitudes',         iret)
        call ncdinq(ncid, idim, name, nr,  iret)
        idim=ncdid(ncid,'number_of_levels',         iret)
        call ncdinq(ncid, idim, name, nth, iret)
ctdf Now nth is the number of eta levels        tdf 10/26/2K4
        write(6,*)'netCDF dimensions ',nr,nc,nth
        ifirst=0 
        ncw=nc
        nrw=nr
        nthw=nth
c
c new version of rdwind, rdwind_name, that reads met variables  
c by name rather than by order. This eliminates the need for   
c                                            tdf 01/05/2K5
        call rdwind(ncid,ncw,nrw,nthw,ifirst)

        DO 10 L = 1,nth
        DO 10 J = 1,NR
          DO 10 I = 1,NC
            IF(ITFLAG.NE.0)THEN
              UVELOLD(J,I,L)=UVELNEW(J,I,L)
              VVELOLD(J,I,L)=VVELNEW(J,I,L)
ctdf Geop. altitude                            tdf 09/09/2K5
              ALTOLD(J,I,L)=ALTNEW(J,I,L)
ctdf p-dot
              WVELOLD(J,I,L)=WVELNEW(J,I,L)
              POLD(J,I,L)=PNEW(J,I,L)
              PVOLD(J,I,L)=PVNEW(J,I,L)
              XMSFOLD(J,I,L)=XMSFNEW(J,I,L)
              QDFOLD(J,I,L)=QDFNEW(J,I,L)
ctdf Add in several new arrays                 tdf 10/29/2K4
              SHOLD(J,I,L)=SHNEW(J,I,L)
              THOLD(J,I,L)=THNEW(J,I,L)
ctdf              THDOLD(J,I,L)=THDNEW(J,I,L)
            ENDIF
c tdf NB Changed ordering of "sim" arrays        tdf 02/04/2K5
            UVELNEW(J,I,L)=usim(I,J,L) 
            VVELNEW(J,I,L)=vsim(I,J,L) 
ctdf geop. altitude
            ALTNEW(J,I,L)=altsim(I,J,L) 
ctdf p-dot
            WVELNEW(J,I,L)=wsim(I,J,L) 
            XMSFNEW(J,I,L)=xmsfsim(I,J,L)
            PNEW(J,I,L)=psim(I,J,L) 
            PVNEW(J,I,L)=pvsim(I,J,L) 
            QDFNEW(J,I,L)=qdfsim(I,J,L)
ctdf Add in several new arrays                 tdf 10/29/2K4
            SHNEW(J,I,L)=shsim(I,J,L)
            THNEW(J,I,L)=thsim(I,J,L)
ctdf            THDNEW(J,I,L)=thdsim(I,J,L)
c tdf NB Changed ordering of "sim" arrays        tdf 02/04/2K5
10      CONTINUE
ctdf surface pressure                          tdf 7/29/2K5
        DO 15 J = 1,NR
          DO 15 I = 1,NC
            IF(ITFLAG.NE.0)THEN
              PSFCOLD(J,I)=PSFCNEW(J,I)
            ENDIF
            PSFCNEW(J,I)=psfcsim(I,J) 
15      CONTINUE
ctdf
        write(6,*)' compute trajectories'
        call flush(6)
c
        IF(ITFLAG.EQ.0) GO TO 998
C
C  COMPUTE TRAJECTORIES
C
        DO 30 ISTEP=1,NSTEP
c
          XSTEP=ISTEP-1      
          TSCALE=(YSTEP-XSTEP)/YSTEP
          wmax=0.
          DO 20 L=1,nth
          DO 20 I=1,NC
            DO 20 J=1,NR
              UGRD(J,I,L) = UVELNEW(J,I,L)+
     &          TSCALE*(UVELOLD(J,I,L)-UVELNEW(J,I,L))
              VGRD(J,I,L) = VVELNEW(J,I,L)+
     &          TSCALE*(VVELOLD(J,I,L)-VVELNEW(J,I,L))
ctdf Geop. altitude                           tdf 09/09/2K5
              ALTGRD(J,I,L) = ALTNEW(J,I,L)+
     &          TSCALE*(ALTOLD(J,I,L)-ALTNEW(J,I,L))
ctdf p-dot (mbar/s)
              WGRD(J,I,L) = WVELNEW(J,I,L)+
     &          TSCALE*(WVELOLD(J,I,L)-WVELNEW(J,I,L))
ctdf p-dot (mbar/s)
              PGRD(J,I,L) = PNEW(J,I,L)+
     &          TSCALE*(POLD(J,I,L)-PNEW(J,I,L))
              PVGRD(J,I,L) = PVNEW(J,I,L)+
     &          TSCALE*(PVOLD(J,I,L)-PVNEW(J,I,L))
              XMSFGRD(J,I,L) = XMSFNEW(J,I,L)+
     &          TSCALE*(XMSFOLD(J,I,L)-XMSFNEW(J,I,L))
              QDFGRD(J,I,L) = QDFNEW(J,I,L)+
     &          TSCALE*(QDFOLD(J,I,L)-QDFNEW(J,I,L))
ctdf Add in several new arrays                 tdf 10/29/2K4
              SHGRD(J,I,L) = SHNEW(J,I,L)+
     &          TSCALE*(SHOLD(J,I,L)-SHNEW(J,I,L))
              THGRD(J,I,L) = THNEW(J,I,L)+
     &          TSCALE*(THOLD(J,I,L)-THNEW(J,I,L))
ctdf              THDGRD(J,I,L) = THDNEW(J,I,L)+
ctdf     &          TSCALE*(THDOLD(J,I,L)-THDNEW(J,I,L))
c ******************************************************
20        CONTINUE
          wmax=sqrt(wmax)
ctdf surface pressure                          tdf 7/29/2K5
          DO 21 I=1,NC
            DO 21 J=1,NR
              PSFCGRD(J,I) = PSFCNEW(J,I)+
     &          TSCALE*(PSFCOLD(J,I)-PSFCNEW(J,I))
21        CONTINUE
C
c  introduce any new DIAL, DC-8, or curtain profiles   tdf 4/18/2K5
c  Use "date" and "dial_date" and time difference to make the test 
          if (dial .or. dc8 .or. curtain .or. calipso) then
c adjust etime up for call to calendar         Duncan 5/12/2000 
            rtime = etime + 0.0001
            call calendar(istime,rtime,ictime)
      print *,' istime, rtime, ictime'
      print *, istime, rtime, ictime
            dtb2 = 0.5*dt/3600.      ! half timestep in hours 
            date = ictime/100 
            itdial_date = dial_date 
      print *,' itdial_date =',itdial_date
            if (dt .lt. 0.0) then    ! back trajectory condition 
   22         continue 
              if (idial.ge.1) then
                tdial = tsave(idial) 
c  adjust for midnight overlap                 Duncan 4/10/2000
                if (tdial .ge. 24.+dtb2) then
      print *,' updating itdial_date'
      print *,' tdial, itdial_date, dial_date'
                  tdial = tdial-24.
                  itdial_date = dial_date+1
                endif
                tdiff = abs(abs(tdial)-abs(gmt))
                write(6,359) ictime, date, itdial_date 
                write(6,360) idial, tdiff, gmt, tdial, ntraj 
                if ((date .eq. itdial_date)
     1             .and. (tdiff .lt. abs(dtb2))) then
ctdf                  if (dial) call init_dial(idial)
                  if (dial) call init_dial
ctdf                  if (dc8) call init_dc8(idial)
                  if (dc8) call init_dc8
ctdf Include Curtain option here              tdf 4/18/2K5
ctdf                  if (curtain) call init_curtain(idial)
                  if (curtain) call init_curtain
ctdf Include CALIPSO option here              tdf 4/22/2K5
ctdf                  if (calipso) call init_calipso(idial)
                  if (calipso) then
                    call init_calipso(ist,last)
ctdf Retrieve p from altitude                 tdf 9/9/2K5
                    call getdiag_calinit(ist,last)
                  endif
                  idial=idial-1
                  goto 22
                endif
              endif
            endif 
            if (dt .ge. 0.0) then    ! forward trajectory condition 
   23         continue 
              if (idial.le.ncount) then
                tdial = tsave(idial) 
c  adjust for midnight overlap                 Duncan 4/10/2000
                if (tdial .ge. 24.-dtb2) then
                  tdial = tdial-24.
                  itdial_date = dial_date+1
                endif
                tdiff = abs(abs(tdial)-abs(gmt))
                write(6,359) ictime, date, itdial_date 
                write(6,360) idial, tdiff, gmt, tdial, ntraj 
                if ((date .eq. itdial_date)
     1             .and. (tdiff .lt. abs(dtb2))) then
ctdf                  if (dial) call init_dial(idial)
                  if (dial) call init_dial
ctdf                  if (dc8) call init_dc8(idial)
                  if (dc8) call init_dc8
ctdf Include Curtain option here              tdf 4/18/2K5
ctdf                  if (curtain) call init_curtain(idial)
                  if (curtain) call init_curtain
ctdf Include CALIPSO option here              tdf 8/22/2K5
ctdf                  if (calipso) call init_calipso(idial)
                  if (calipso) then
                    call init_calipso(ist,last)
ctdf Retrieve p from altitude                 tdf 9/9/2K5
                    call getdiag_calinit(ist,last)
                  endif
                  idial=idial+1
                  goto 23
                endif 
              endif
            endif
          endif                      ! end of (dial / dc8) condition 
  359 format (' ictime = ',I10,' date = ',I10,' dial_date = ',I10)
  360 format (' # ',I5,' tdiff ',F10.4,' model gmt ',
     1         F10.4,' dial time ',F10.4,' ntraj =',I6)
  361 format (' idial',I4,' istime = ',I10,' etime = ',F10.4,
     1        ' ictime = ',I10)
C
c  DIAL initialisation complete for this timestep     Duncan 6/28/99 
c
          print *,' calling traject'
          CALL TRAJECT
c
c compute solar zenith angle for each parcel and update 
c fraction of time spent in daylight and nighttime          Duncan 4/10/97
          call solar_fraction(doy,gmt,xn,yn)
c
c Move call to getdiag out into the timestep loop     Duncan 7/18/2000
          call getdiag
c
          etime=etime+dt/3600.
          wtime=wtime+dt/3600.
c 
c Use the following conditions when the parcel initialisation
c is staggered with respect to the start of the simulation. Duncan 7/1/99 
c e.g. when initializing at 12Z with 0Z winds ......
cDD         if (abs(etime) .gt. 11.9 .AND. abs(etime) .lt. 12.1) then
cDD            call initial
cDD            call jul_gmt (ictime,julday,gmt)
cDD         endif
c
c update gmt and julday after each timestep              Duncan 4/10/97
          gmt = gmt + dt/3600.
ctdf Changed following line to avoid rounding problems  5/10/2K5
ctdf          if (gmt .lt. 0.0) then                
          if (gmt .lt. -0.001) then                
            print *,' gmt updated from ',gmt       ! tdf 5/10/2K5
            julday = julday - 1
            gmt = 24. + gmt
            print *,' gmt updated to   ',gmt       ! tdf 5/10/2K5
          endif
          if (gmt .gt. 24.) then
            julday = julday + 1
            gmt = gmt - 24.
            print *,' gmt updated to ',gmt       ! tdf 5/10/2K5
          endif
          if (julday .gt. 365) then
            julday = julday - 365
          endif
          if (julday .lt. 0) then
            julday = 365 - julday
          endif
c         write(6,*)'completed ',etime,' hours',
c    1              ' jday =',julday,' gmt =',gmt
c
c  test for time to write to output file
c
          if ( (abs(wtime).lt.0.1) .or. 
     +         (abs (abs(wtime)-abs(dtout)) .lt. 0.1) ) then
            if (ntraj .lt. 1) then
c Round etime to whole number of hours when output condition
c satisfied. This should regularly correct any truncation error   
c in etime caused by successive additions of dt.  Duncan 5/12/2000
              etime = nint(etime)*1.0
c adjust etime up for call to calendar         Duncan 5/12/2000 
              rtime = etime + 0.0001
              call calendar(istime,rtime,ictime)
              write(6,*)'current time is',ictime
              print *,' No trajectories found at output time'
            else
c
c round etime to whole number of hours when output condition
c satisfied. This should regularly correct any truncation error   
c in etime caused by successive additions of dt.  Duncan 5/12/2000
              etime = nint(etime)*1.0
c adjust etime up for call to calendar         Duncan 5/12/2000 
              rtime = etime + 0.0001
              call calendar(istime,rtime,ictime)
              write(6,*)'writing output, ntraj =',ntraj
              write(6,*)'istime ',istime,' etime ',etime,
     1                  ' current time ',ictime
              call write_output(12)
c refresh daylit and nighttime fractions              Duncan 4/10/97
ctdf              do itraj = 1, ntraj
ctdf                frday(itraj) = 0.0
ctdf                frnght(itraj) = 0.0
ctdf              end do
c refresh minimum temperature                         Duncan 7/18/2000
ctdf              do itraj = 1, ntraj
ctdf                xmint(itraj) = 99999.
ctdf              end do 
            endif
            wtime=0.0
          endif
c
30      CONTINUE
        close(10)
998     ITFLAG=1
999   CONTINUE		
      STOP
      END 

      subroutine initial
      include 'comm_gfs_3d_v01.'
c
c     initialise trajectories
c
      ntraj=0
c
c  initialize streamlines
c
c new streamline initialisation based on stream function  Duncan 10/13/99
c      if (stream) then 
c         ciso = strm0 * 1.0e-6 
c         ds = ds1
c         do j = 1, nstrm
c           call isopleth2(ciso,ds)
c           write(6,*)'line',j,'sf =',ciso,' ntraj=',ntraj
c           ciso = ciso + dstrm * 1.0e-6 
c         end do
c      endif 
c 
c initialize Potential Vorticity Isopleths
c 
      if(pviso) then 
c
c  smooth pv at poles
c
        pvnp=0.
        pvsp=0.
        do l=1,nth
        do i=1,nc 
c tdf NB Changed ordering of "sim" arrays        tdf 02/04/2K5
          pvsp=pvsp+pvsim(i,2,l)/nc
          pvnp=pvnp+pvsim(i,nr-1,l)/nc
        enddo
        do i=1,nc 
c tdf NB Changed ordering of "sim" arrays        tdf 02/04/2K5
          pvsim(i,1,l)=pvsp
          pvsim(i,nr,l)=pvnp
        enddo
        enddo
        ciso=pv0
        ds=ds2
ctdf        do n=1,npviso
ctdf          call isopleth2(ciso,ds)
ctdf          ciso=ciso+dpv
ctdf        enddo
      endif
c
c initialize Temperature Isopleths
c 
      if(tiso) then 
        ciso=temp0
        ds=ds3
c        do n=1,ntiso
c          call isopleth2(ciso,ds)
c          write(6,*)'line',n,'temp=',ciso,' ntraj=',ntraj
c          ciso=ciso+dtemp
c        enddo
      endif
c
c initialize along ER-2 flight path or DIAL curtain
c 
      if(er2) call flight_path(theta)
c
c  read in dial data for the start day              Duncan 6/28/99  
      if (dial) call read_dial  
c
c  read in dc-8 annulus for the start day           VLH 9/10/2002
      if (dc8) call read_dc8_annulus
c
c  read in xyt data for curtain start day           tdf 4/18/2K5 
      if (curtain) call read_curtain 
c
c  read in calipso data for the start day           tdf 8/19/2K5  
      if (calipso) call read_calipso_dat
c
c initialize space filling
c 
      if(space) call spacefill
c
c initialize Henry's trajectories
c 
      if(wildfire) call wildfire_fill
c
c initialize filling of ER2 range ring
c 
      if(range_fill) call er2_range_fill
c
c initialise daylit and nighttime fractions              Duncan 4/10/97
      do itraj = 1, ntraj
ctdf        frday(itraj) = 0.0
ctdf        frnght(itraj) = 0.0
c Adding t0n and agen variables                          Duncan 11/05/99 
        t0n(itraj) = 0.0 
        agen(itraj) = 0.0
c Adding xmint variable                                  Duncan 7/18/2000 
ctdf        xmint(itraj) = 99999.
      end do
c   
      print *,'initialisation completed'
      return
      end
      subroutine jul_gmt (istime,julday,gmt)
c
c*****************************************************************c
c  From the start time, istime, compute the Julian day and GMT    c
c  istime is in "yymmddhh" format.                                c
c                                             Duncan 4/10/97      c
c*****************************************************************c
c
      DIMENSION MNO(12)        
      integer*4 istime,ictime,inch,incd,incm,incy
      integer*4 yy1,mm1,dd1,hh1,yy2,mm2,dd2,hh2
      data (MNO(I),I=1,12)/31,28,31,30,31,30,31,31,30,31,30,31/    
C                                                                       
      yy1=istime/1000000
      mm1=istime/10000-yy1*100
      dd1=istime/100-yy1*10000-mm1*100
      hh1=istime-yy1*1000000-mm1*10000-dd1*100
c
      print *,'Jul_gmt ',yy1,mm1,dd1,hh1
      julday = dd1
      if (mm1 .gt. 1) then
        do m = 1, mm1-1
          julday = julday + mno(m)
        end do
      endif
      gmt = 1.0 * hh1
      print *,'Jul_gmt ',julday,gmt
      return
      end
      subroutine solar_fraction
c*****************************************************************c
c  Compute solar zenith angle for each parcel and update          c
c  daylit and nighttime fractions                                 c
c  frac_time is the fraction of an output time interval in each   c
c  timestep. julday (Julian day) and gmt (Greenwich Mean Time)    c
c  are continually updated and included in 'comm_ashoe_3d_v08.    c
c                                     Duncan 4/10/97              c
c*****************************************************************c
      include 'comm_gfs_3d_v01.'
      if (ntraj .ge. 1) then
        frac_time = abs(dt) / (3600.*dtout)
        doy = 1. * julday
        do itraj = 1, ntraj
          rlat = yn(itraj)
          rlon = xn(itraj)
          call zenith_angle(doy,gmt,rlat,rlon,chi)
ctdf          if (chi .le. 90) then
ctdf            frday(itraj) = frday(itraj) + frac_time
ctdf          endif
ctdf          if (chi .gt. 90) then
ctdf            frnght(itraj) = frnght(itraj) + frac_time
ctdf          endif
        end do
      endif
      return
      end
      subroutine zenith_angle(doy,gmt,rlat,rlon,chi)
c*****************************************************************c
c  Compute solar zenith angle for each parcel                     c
c                                                                 c
c INPUT:                                                          c
c  doy   : julian day (INPUT)                                     c
c  gmt   : greenwich mean time (INPUT)                            c
c  tlat  : true latitude  (INPUT)                                 c
c  tlon  : true longitude (INPUT)                                 c
c OUTPUT:                                                         c
c  chi   : zenith angle (degrees) at position (tlat,tlon)         c
c*****************************************************************c
c
c  zenith angle computation
      pi=3.14159265
      dtor=pi/180.
      earinc=23.5
c
      sinlat=sin(rlat*dtor)
      coslat=sqrt(1.-sinlat**2)
      sinlon=sin(rlon*dtor)
      coslon=cos(rlon*dtor)
      soya=(doy-81.25)*pi/182.5           ! day angle
      soha=2.*pi*(gmt-12.)/24.            ! hour angle
      soha=-soha
      sininc=sin(earinc*dtor)
      sindec=sininc*sin(soya)
      cosdec= sqrt(1.-sindec**2)
      coszen=cos(soha)*coslon+sin(soha)*sinlon
      coszen=coszen*cosdec*coslat
      coszen=sindec*sinlat+coszen
      coszen=min(max(coszen,-1.),1.)
      chi = acos(coszen)
      chi = chi/dtor
      return
      end

      subroutine write_output(iout)
c
c*****************************************************************c
c                                                                 c
c  Write to output file, unit iout                                c
c                                                                 c
c*****************************************************************c
c
      include 'comm_gfs_3d_v01.'
      print *,'write_output ',istime,ictime,etime,ntraj
      WRITE(iout)istime,ictime,etime,ntraj
      write(iout)'xtraj'
      WRITE(iout)(xn(j),j=1,ntraj)
      write(iout)'ytraj'
      WRITE(iout)(yn(j),j=1,ntraj)
      write(iout)'ptraj'
      WRITE(iout)(pn(j),j=1,ntraj)
      write(iout)'pvtrj'
      WRITE(iout)(pvn(j),j=1,ntraj)
ctdf add in potential temperature                tdf 11/05/2K4
      write(iout)'thtraj'
      WRITE(iout)(thn(j),j=1,ntraj)
ctdf thdn is theta-dot                           tdf 11/05/2K4
ctdf      write(iout)'thdtraj'
ctdf      WRITE(iout)(thdn(j),j=1,ntraj)
      write(iout)'qdftraj'
      WRITE(iout)(qdfn(j),j=1,ntraj)
      write(iout)'xmsftraj'
      WRITE(iout)(xmsfn(j),j=1,ntraj)
ctdf Specific humidity                           tdf 11/05/2K4
      write(iout)'shtraj'
      WRITE(iout)(shn(j),j=1,ntraj)
ctdf Backscatter                                 tdf 11/05/2K4
ctdf      write(iout)'bstraj'
ctdf      WRITE(iout)(bsn(j),j=1,ntraj)
c Adding t0n and agen variables                  Duncan 11/05/99
      write(iout)'t0_traj'
      WRITE(iout)(t0n(j),j=1,ntraj)
      write(iout)'age_traj'
      WRITE(iout)(agen(j),j=1,ntraj)
c altitude                                       tdf 9/9/2K5
      write(iout)'alt_traj'
      WRITE(iout)(altn(j),j=1,ntraj)
      return
      end

      subroutine read_list_header(iunit)
c
c*****************************************************************c
c                                                                 c
c  Read header information, list directed, from unit iunit        c
c                                                                 c
c*****************************************************************c
c
      include 'comm_gfs_3d_v01.'
      read(iunit,*)charexp
      read(iunit,*)nthp
      read(iunit,*)(thp(k),k=1,nthp)
      read(iunit,*)ukmo
      read(iunit,*)nmc
      read(iunit,*)ecmwf
      read(iunit,*)restart
      read(iunit,*)rfile
      read(iunit,*)irtime
      read(iunit,*)dir
      read(iunit,*)nfiles
      do n=1,nfiles
         read(iunit,*)ifile(n)
         print *, n,nfiles
      enddo
      read(iunit,*)istime
      read(iunit,*)dtflow
      read(iunit,*)dt
      read(iunit,*)igw
      read(iunit,*)stream
      read(iunit,*)nstrm
      read(iunit,*)ds1
c choose a lowest stream function value and increment
      read(iunit,*)strm0
      read(iunit,*)dstrm
      read(iunit,*)pviso
      read(iunit,*)npviso
      read(iunit,*)ds2
      read(iunit,*)pv0
      read(iunit,*)dpv
      read(iunit,*)tiso
      read(iunit,*)ntiso
      read(iunit,*)ds3
      read(iunit,*)temp0
      read(iunit,*)dtemp
      read(iunit,*)space
      read(iunit,*)dxs
      read(iunit,*)dys
ctdf New option wildfire                      tdf 2/4/2K5
      read(iunit,*)wildfire
      read(iunit,*)dirh
      read(iunit,*)hfile
ctdf New option wildfire                      tdf 2/4/2K5
      read(iunit,*)range_fill
      read(iunit,*)stlat
      read(iunit,*)stlon
      read(iunit,*)er2_range
      read(iunit,*)nrings
      read(iunit,*)npts
      read(iunit,*)dial
      read(iunit,*)dird
      read(iunit,*)ndial
      do n=1,ndial
         read(iunit,*)dfile(n)
         print *, n, dfile(n)
      enddo
ctdf New option model curtain              tdf 4/18/2K5
      read(iunit,*)curtain
      print *, ' curtain =', curtain
      read(iunit,*)dirc
      print *, ' dirc =', dirc
      read(iunit,*)ncurt
      print *, ' ncurt =', ncurt
      do n=1,ncurt
         read(iunit,*)cfile(n)
         print *, n, cfile(n)
      enddo
ctdf New option model calipso              tdf 8/22/2K5
      read(iunit,*)calipso
      print *, 'calipso =', calipso
      read(iunit,*)dirc
      print *, ' dirc =', dirc
      read(iunit,*)ncurt
      print *, ' ncurt =', ncurt
      do n=1,ncurt
         read(iunit,*)cfile(n)
         print *, n, cfile(n)
      enddo
ctdf 
      read(iunit,*)er2
      read(iunit,*)nleg
      do n=1,nleg
         read(iunit,*)xleg0(n)
         read(iunit,*)yleg0(n)
         read(iunit,*)xleg1(n)
         read(iunit,*)yleg1(n)
         print *, n, nleg
      enddo
      read(iunit,*)dc8
      read(iunit,*)dirdc8
      read(iunit,*)ndc8
      do n=1,ndc8
         read(iunit,*)dc8file(n)
         print *, n, dc8file(n)
      enddo
      read(iunit,*)dtout
      read(iunit,*)ofile
      print *, ofile
      return
      end

      subroutine write_list_header(iunit)
c
c*****************************************************************c
c                                                                 c
c  Write header information, list directed, to unit iunit         c
c                                                                 c
c*****************************************************************c
c
      include 'comm_gfs_3d_v01.'
      write(iunit,*)charexp
      write(iunit,*)nthp
      write(iunit,*)(thp(k),k=1,nthp) 
      write(iunit,*)ukmo
      write(iunit,*)nmc
      write(iunit,*)ecmwf
      write(iunit,*)restart
      write(iunit,*)rfile
      write(iunit,*)irtime
      write(iunit,*)dir
      write(iunit,*)nfiles
      do n=1,nfiles
         write(iunit,*)ifile(n)
      enddo
      write(iunit,*)istime
      write(iunit,*)dtflow
      write(iunit,*)dt
      write(iunit,*)igw
      write(iunit,*)stream
      write(iunit,*)nstrm
      write(iunit,*)ds1
c choose a lowest stream function value and increment
      write(iunit,*)strm0                           
      write(iunit,*)dstrm                           
      write(iunit,*)pviso
      write(iunit,*)npviso
      write(iunit,*)ds2
      write(iunit,*)pv0
      write(iunit,*)dpv
      write(iunit,*)tiso
      write(iunit,*)ntiso
      write(iunit,*)ds3
      write(iunit,*)temp0
      write(iunit,*)dtemp
      write(iunit,*)space
      write(iunit,*)dxs
      write(iunit,*)dys
ctdf New option wildfire                      tdf 2/4/2K5
      write(iunit,*)wildfire
      write(iunit,*)dirh
      write(iunit,*)hfile
ctdf New option wildfire                      tdf 2/4/2K5
      write(iunit,*)range_fill
      write(iunit,*)stlat
      write(iunit,*)stlon
      write(iunit,*)er2_range
      write(iunit,*)nrings
      write(iunit,*)npts
      write(iunit,*)dial
      write(iunit,*)dird
      write(iunit,*)ndial
      do n=1,ndial
         write(iunit,*)dfile(n)
      enddo
ctdf New option model curtain              tdf 4/18/2K5
      write(iunit,*)curtain
      write(iunit,*)dirc
      write(iunit,*)ncurt
      do n=1,ncurt
         write(iunit,*)cfile(n)
      enddo
ctdf New option model calipso              tdf 8/22/2K5
      write(iunit,*)calipso
      write(iunit,*)dirc
      write(iunit,*)ncurt
      do n=1,ncurt
         write(iunit,*)cfile(n)
      enddo
ctdf
      write(iunit,*)er2
      write(iunit,*)nleg
      do n=1,nleg
         write(iunit,*)xleg0(n)
         write(iunit,*)yleg0(n)
         write(iunit,*)xleg1(n)
         write(iunit,*)yleg1(n)
      enddo
      write(iunit,*)dc8
      write(iunit,*)dirdc8
      write(iunit,*)ndc8
      do n=1,ndc8
         write(iunit,*)dc8file(n)
      enddo
      write(iunit,*)dtout
      write(iunit,*)ofile
      return
      end

      subroutine write_unf_header(iunit)
c
c*****************************************************************c
c                                                                 c
c  Write header information, unformatted, to output file          c
c                                                                 c
c*****************************************************************c
c
      include 'comm_gfs_3d_v01.'
      write(iunit)charexp                         
      write(iunit)nthp
      write(iunit)(thp(k),k=1,nthp)
      write(iunit)ukmo                            
      write(iunit)nmc                             
      write(iunit)ecmwf                           
      write(iunit)restart
      write(iunit)rfile
      write(iunit)irtime
      write(iunit)dir                             
      write(iunit)nfiles                          
      do n=1,nfiles                           
         write(iunit)ifile(n)                        
      enddo                                    
      write(iunit)istime
      write(iunit)dtflow                          
      write(iunit)dt                              
      write(iunit)igw                             
      write(iunit)stream                          
      write(iunit)nstrm                           
      write(iunit)ds1                             
c choose a lowest stream function value and increment
      write(iunit)strm0                           
      write(iunit)dstrm                           
      write(iunit)pviso                           
      write(iunit)npviso                          
      write(iunit)ds2                             
      write(iunit)pv0                             
      write(iunit)dpv                             
      write(iunit)tiso                            
      write(iunit)ntiso                           
      write(iunit)ds3                             
      write(iunit)temp0                           
      write(iunit)dtemp                           
      write(iunit)space                           
      write(iunit)dxs                             
      write(iunit)dys                             
ctdf New option wildfire                      tdf 2/4/2K5
      write(iunit)wildfire
      write(iunit)dirh
      write(iunit)hfile
ctdf New option wildfire                      tdf 2/4/2K5
      write(iunit)range_fill
      write(iunit)stlat
      write(iunit)stlon
      write(iunit)er2_range
      write(iunit)nrings
      write(iunit)npts
      write(iunit)dial
      write(iunit)dird
      write(iunit)ndial
      do n=1,ndial
         write(iunit)dfile(n)
      enddo
ctdf New option model curtain              tdf 4/18/2K5
      write(iunit)curtain
      write(iunit)dirc
      write(iunit)ncurt
      do n=1,ncurt
         write(iunit)cfile(n)
      enddo
ctdf New option model calipso              tdf 8/22/2K5
      write(iunit)calipso
      write(iunit)dirc
      write(iunit)ncurt
      do n=1,ncurt
         write(iunit)cfile(n)
      enddo
ctdf New option model curtain              tdf 4/18/2K5
      write(iunit)er2                             
      write(iunit)nleg                            
      do n=1,nleg                              
        write(iunit)xleg0(n)                        
        write(iunit)yleg0(n)                        
        write(iunit)xleg1(n)                        
        write(iunit)yleg1(n)                        
      enddo                                    
      write(iunit)dc8
      write(iunit)dirdc8
      write(iunit)ndc8
      do n=1,ndc8
         write(iunit)dc8file(n)
      enddo
      write(iunit)dtout                           
      write(iunit)ofile
      return
      end

      subroutine read_restart_data(iunit)
c
c*****************************************************************c
c                                                                 c
c   Read restart data, unformatted, from unit iunit               c
c                                                                 c
c*****************************************************************c
c
      include 'comm_gfs_3d_v01.'
      character*5 blank
c
c  first skip over header information in restart unit
c
      do 10 k = 1, 9
   10   read(iunit)
      read(iunit)nrf
c      write (6,*) ' nrf = ',nrf
      do n=1,nrf
        read(iunit)
      enddo
      do 20 k = 1, 23
   20   read(iunit)
      read(iunit)nrl
c      write (6,*) ' nrl = ',nrl
      do n=1,nrl
        read(iunit)
        read(iunit)
        read(iunit)
        read(iunit)
      enddo
      read(iunit)dtout
      read(iunit)ofile
c
c skip to restart day
c
   40 continue
      write(6,*)'reading restart data '
      READ(iunit) istime,ictime,time,ntraj
      write (6,*) 'data found for time',ictime
      read(iunit) blank
      write(6,*)blank
      READ(iunit) (xn(j),j=1,ntraj)
      read(iunit) blank
      write(6,*)blank
      READ(iunit) (yn(j),j=1,ntraj)
      read(iunit) blank
      write(6,*)blank
      READ(iunit) (pn(j),j=1,ntraj)
      read(iunit) blank
      write(6,*)blank
      READ(iunit) (pvn(j),j=1,ntraj)
ctdf potential temperature                        tdf 11/05/2K4
      read(iunit) blank
      write(6,*)blank
      READ(iunit) (thn(j),j=1,ntraj)
      read(iunit) blank
      write(6,*)blank
      READ(iunit) (thdn(j),j=1,ntraj)
      read(iunit) blank
      write(6,*)blank
      READ(iunit) (qdfn(j),j=1,ntraj)
      read(iunit) blank
      write(6,*)blank
      READ(iunit) (xmsfn(j),j=1,ntraj)
ctdf specific humidity                            tdf 11/05/2K4
      read(iunit) blank
      write(6,*)blank
      READ(iunit) (shn(j),j=1,ntraj)
ctdf Backscatter                                  tdf 07/12/2K5
c      read(iunit) blank
c      write(6,*)blank
c      READ(iunit) (bsn(j),j=1,ntraj)
      read(iunit) blank
      write(6,*)blank
      READ(iunit) (t0n(j),j=1,ntraj)
      read(iunit) blank
      write(6,*)blank
      READ(iunit) (agen(j),j=1,ntraj)
      if (ictime .lt. irtime-1) goto 40
      if (ictime .gt. irtime+1) then
        write (6,*) 'error: restart data for time ',irtime
        write (6,*) 'not found in restart file'
        stop
      endif
C
      return
      end


      subroutine rdwind(ncid,ncw,nrw,nthw,ifirst)
c
c*****************************************************************c
c new version of rdwind, rdwind_name, that reads met variables    c
c by name rather than by order. This eliminates the need for      c
c arr1, arr2, arr3, etc.                   tdf 02/04/2K5          c
c                                                                 c
c   Read flow field data                                          c
ctdf Vector order for new eta-coordinate data                     c
c 1-d vectors    diagnostic                    label   previously c
c       alon   ; longitudes                  longitudes   arr1    c
c       alat   ; latitudes                   latitudes    arr2    c
c ignore etalev   tdf 07/12/2K5
c       etalev ; eta levels (units of theta) eta_levels   arr3    c
c 3-d arrays     diagnostic                    label   previously c
c       ipv    ; IPV                           ipv        arr4    c
c       prs    ; pressure          (mbar)      press      arr5    c
c       alt    ; Geop. altitude    (m)                            c
c       msf    ; Montgomery S.F.   (m2/s2)     msf        arr6    c
c       u      ; zonal wind        (m/s)       u_wind     arr7    c
c       v      ; meridional wind   (m/s)       v_wind     arr8    c
c       theta  ; potential temperature (K)     theta      arr9    c
c       shm    ; specific humidity (kg/kg)     sp_hum     arr10   c
c       qdf    ; QDF - strain rotation         qdf        arr11   c
c       etadot ; eta-dot           (Pa/sec)    etadot     arr12   c
c       thdot  ; theta-dot         (K/sec)     th_dot     arr13   c
c                                                                 c
c Arrays now called by name, rather than by order tdf 01/05/05    c
c*****************************************************************c
c
      include 'comm_gfs_3d_v01.'
      integer istart(3),icount(3),ict2(2)
      data istart/1,1,1/

ctdf      write(6,*)' READING 3D UKMO ISENTROPIC DATA ......'
      write(6,*)' READING GFS pressure coordinate ......'

c Note new ordering nc, nr, nth          tdf 2/4/2K4
      icount(1)=ncw
      icount(2)=nrw
      icount(3)=nthw
c
      ict2(1)=ncw
      ict2(2)=nrw

ctdf call by name                             tdf 01/05/05
c
      idum=ncvid(ncid,'longitudes'    ,iret)
c      print *,' idum =',idum,' iret =',iret
      call ncvgt(ncid, idum, 1, ncw, x, iret)
c      print *,(x(k),k=1,6)
      call flush(6)
c
      idum=ncvid(ncid,'latitudes'    ,iret)
c      print *,' idum =',idum,' iret =',iret
      call ncvgt(ncid, idum, 1, nrw, y, iret)
c      print *,(y(k),k=1,6)
      call flush(6)
c
      idum=ncvid(ncid,'eta_levels'    ,iret)    ! eta levels  ?
c      print *,' idum =',idum,' iret =',iret
      call ncvgt(ncid, idum, 1, nthw, thw, iret)
c      print *,thw
      call flush(6)
c
      idum=ncvid(ncid,'ipv'    ,iret)
c      print *,' ipv =',idum,' iret =',iret
      call ncvgt(ncid, idum, istart, icount, pvsim, iret)
c      print *,'(pvsim(1,60,k),k=nthw-17,nthw)'
c      print *,(pvsim(1,60,k),k=nthw-17,nthw)
      call flush(6)
c
      idum=ncvid(ncid,'press'    ,iret)
c      print *,' ipr =',idum,' iret =',iret
      call ncvgt(ncid, idum, istart, icount, psim, iret)
c      print *,'(psim(1,60,k),k=nthw-17,nthw)'
c      print *,(psim(1,60,k),k=nthw-17,nthw)
      call flush(6)
c
ctdf geop. altitude
      idum=ncvid(ncid,'height'    ,iret)
c      print *,' ipa =',idum,' iret =',iret
      call ncvgt(ncid, idum, istart, icount, altsim, iret)
c      print *,'(altsim(1,60,k),k=nthw-17,nthw)'
c      print *,(altsim(1,60,k),k=nthw-17,nthw)
      call flush(6)
c
c Montgomery stream function
      idum=ncvid(ncid,'msf'    ,iret)
c      print *,' imsf =',idum,' iret =',iret
      call ncvgt(ncid, idum, istart, icount, xmsfsim, iret)
c
      idum=ncvid(ncid,'u_wind'    ,iret)
c      print *,' iuu =',idum,' iret =',iret
      call ncvgt(ncid, idum, istart, icount, usim, iret)
c
      idum=ncvid(ncid,'v_wind'    ,iret)
c      print *,' ivv =',idum,' iret =',iret
      call ncvgt(ncid, idum, istart, icount, vsim, iret)
c
c Potential temperature
      idum=ncvid(ncid,'theta'    ,iret)     ! theta
c      print *,' ith =',idum,' iret =',iret
      call ncvgt(ncid, idum, istart, icount, thsim, iret)
c      print *,'(thsim(1,60,k),k=nthw-17,nthw)'
c      print *,(thsim(1,60,k),k=nthw-17,nthw)
      call flush(6)
c
      idum=ncvid(ncid,'sp_hum'    ,iret)   ! specific humidity 
c      print *,' ish =',idum,' iret =',iret
      call ncvgt(ncid, idum, istart, icount, shsim, iret)
c
      idum=ncvid(ncid,'qdf'    ,iret)
      print *,' iqdf =',idum,' iret =',iret
      call ncvgt(ncid, idum, istart, icount, qdfsim, iret)
c
      idum=ncvid(ncid,'p_dot'    ,iret)
c      print *,' ipd =',idum,' iret =',iret
      call ncvgt(ncid, idum, istart, icount, wsim, iret)  ! dp/dt
c
      idum=ncvid(ncid,'psfc'    ,iret)
c      print *,' ips =',idum,' iret =',iret
      call ncvgt(ncid, idum, istart, ict2, psfcsim , iret)  ! psfc
c
c theta-dot not included
      print *,' input fields read OK'
ctdf
      call ncclos(ncid, iret)
ctdf
      write(6,*)' all arrays read OK'
      call flush(6)
C
      dlon=x(2)-x(1)
      dlat=y(2)-y(1)
ctdf
c      write(6,*)' alons',x
c      write(6,*)' alats',y
ctdf      write(6,*)' eta_levs',thw
      call flush(6)
ctdf
      write(6,*) 'psfcsim(1,j)'
      write(6,*) (psfcsim(1,j),j=1,nr)
c      write(6,*) 'psim(1,j,1)'
c      write(6,*) (psim(1,j,1),j=1,nr)
c      write(6,*) 'xmsfsim(1,j,1)'
c      write(6,*) (xmsfsim(1,j,1),j=1,nr)
c      write(6,*) 'usim(1,j,1)'
c      write(6,*) (usim(1,j,1),j=1,nr)
c      write(6,*) 'vsim(1,j,1)'
c      write(6,*) (vsim(1,j,1),j=1,nr)
c      write(6,*) 'thsim(1,j,1)'
c      write(6,*) (thsim(1,j,1),j=1,nr)
c      write(6,*) 'shsim(1,j,1)'
c      write(6,*) (shsim(1,j,1),j=1,nr)
c      write(6,*) 'qdfsim(1,j,1)'
c      write(6,*) (qdfsim(1,j,1),j=1,nr)
c      write(6,*) 'wsim(1,j,1)'
c      write(6,*) (wsim(1,j,1),j=1,nr)
      call flush(6)
      return
      end


      subroutine rdwind_old(ncid,arr1,arr2,arr3,arr4,arr5,
     1                  arr6,arr7,arr8,arr9,arr10,arr11,
     2                  arr12,arr13,arr14,arr15,arr16,
     3                  arr17,arr18,
     4                 nrw,ncw,nthw,ifirst)
c dummy routine: see GFS3d_traject_3d_v19.f for this routine
      return
      end


      subroutine getdiag
c
c*****************************************************************c
c                                                                 c
c   Diagnose PV, theta, specific humidity, and QDF                c
c   for each air parcel (GMAO4 eta data case tdf 7/12/2K5)        c
c   also diagnose eta (zn)                                        c
c   INPUT: xn, yn, pn, ps (diagnosed)                             c
c   OUTPUT: pvn, thn, shn, qdfn, zn                               c
c                                                                 c
c*****************************************************************c
c
      include 'comm_gfs_3d_v01.'
      real xmsfj1,xmsfjp1,xmsfj2,xmsfjp2,xmsf1,xmsf2
      real pw(nth), zw(nth)
      real lpnmc(nth)
      lpnmc(1)=      10.0000
      lpnmc(2)=      20.0000
      lpnmc(3)=      30.0000
      lpnmc(4)=      50.0000
      lpnmc(5)=      70.0000
      lpnmc(6)=      100.000
      lpnmc(7)=      150.000
      lpnmc(8)=      200.000
      lpnmc(9)=      250.000
      lpnmc(10)=     300.000
      lpnmc(11)=     350.000
      lpnmc(12)=     400.000
      lpnmc(13)=     450.000
      lpnmc(14)=     500.000
      lpnmc(15)=     550.000
      lpnmc(16)=     600.000
      lpnmc(17)=     650.000
      lpnmc(18)=     700.000
      lpnmc(19)=     750.000
      lpnmc(20)=     800.000
      lpnmc(21)=     850.000
      lpnmc(22)=     900.000
      lpnmc(23)=     925.000
      lpnmc(24)=     950.000
      lpnmc(25)=     975.000
      lpnmc(26)=     1000.00

      nthw = nth
      do 50 n=1,ntraj
        xp=xn(n)
        yp=yn(n)
        pp=pn(n)
        if(xp.lt.x(1))xp=xp+360.

c First, need to establish xscale and yscale
c        do 40 l=1,nthw-1
c          lp1=l+1
cctdf thw are now the eta levels
c          thl=thw(l)
c          thlp1=thw(lp1)
cc VLH 5/2/02         if((zp.le.thl).and.(zp.gt.thlp1))then
c          if((zp.le.thl).and.(zp.ge.thlp1))then
c          zscale=(zp-thl)/(thlp1-thl)

          do 40 i=1,nc
            ip1=i+1
            if(i.eq.nc)ip1=1
            xlon=x(i) 
            xlonp1=x(ip1)
            if(i.eq.nc)xlonp1=360.+x(ip1)
c VLH 5/2/02            if((xp.ge.xlon).and.(xp.lt.xlonp1))then
            if((xp.ge.xlon).and.(xp.le.xlonp1))then
            xscale=(xp-xlon)/(xlonp1-xlon)
            do 30 j=1,nr-1
              jp1=j+1
              xlat=y(j)
              xlatp1=y(jp1)
c
c note: does not interpolate across the poles	VLH 5/8/02
c
              if((yp.ge.xlat).and.(yp.le.xlatp1))then
c DF 10/16/97           if((yp.ge.xlat).and.(yp.lt.xlatp1))then
                yscale=(yp-xlat)/(xlatp1-xlat)

ctdf Now have xscale, yscale for chosen parcel, n
ctdf Find surface pressure at chosen location
                psj=psfcgrd(j,i)+
     +              xscale*(psfcgrd(j,ip1)-psfcgrd(j,i))
                psjp=psfcgrd(jp1,i)+
     +              xscale*(psfcgrd(jp1,ip1)-psfcgrd(jp1,i))
                ps=psj+yscale*(psjp-psj)

ctdf Now construct pressure and eta profiles at location
                do l = 1, nthw
                  pw(l) = lpnmc(l)
                  zw(l) = lpnmc(l)
                end do
ctdf Pressure profile constructed at location
                do l = 1, nthw-1
                  lp1 = l+1
                  pl = pw(l)
                  plp1 = pw(lp1)
ctdf NB top to bottom ordering means that pw(lp1)>pw(l)
ctdf                  if((pp.le.pl).and.(pp.ge.plp1))then
                  if((pp.ge.pl).and.(pp.le.plp1))then
ctdf eta values
                    z1 = zw(l)
                    z2 = zw(lp1)
ctdf eta values
ctdf linear in p    zscale=(pp-pl)/(plp1-pl)
ctdf linear log(p)
                    zscale=alog(pp/pl)/alog(plp1/pl)
ctdf Now have xscale, yscale, zscale for chosen parcel, n
c level l
                pvj1=pvgrd(j,i,l)+xscale*(pvgrd(j,ip1,l)-pvgrd(j,i,l))
                 wj1=wgrd(j,i,l)+xscale*(wgrd(j,ip1,l)-wgrd(j,i,l))
              xmsfj1=xmsfgrd(j,i,l)
     +              +xscale*(xmsfgrd(j,ip1,l)-xmsfgrd(j,i,l))
ctdf potential temperature
              thj1=thgrd(j,i,l)
     +              +xscale*(thgrd(j,ip1,l)-thgrd(j,i,l))
ctdf specific humidity
              shj1=shgrd(j,i,l)
     +              +xscale*(shgrd(j,ip1,l)-shgrd(j,i,l))
c QDF
                 dj1=qdfgrd(j,i,l)
     +              +xscale*(qdfgrd(j,ip1,l)-qdfgrd(j,i,l))
c altitude                                    tdf 9/9/2K5
                 altj1=altgrd(j,i,l)
     +              +xscale*(altgrd(j,ip1,l)-altgrd(j,i,l))

               pvjp1=pvgrd(jp1,i,l)
     +              +xscale*(pvgrd(jp1,ip1,l)-pvgrd(jp1,i,l))
                wjp1=wgrd(jp1,i,l)
     +              +xscale*(wgrd(jp1,ip1,l)-wgrd(jp1,i,l))
             xmsfjp1=xmsfgrd(jp1,i,l)
     +              +xscale*(xmsfgrd(jp1,ip1,l)-xmsfgrd(jp1,i,l))
ctdf potential temperature
                thjp1=thgrd(jp1,i,l)
     +              +xscale*(thgrd(jp1,ip1,l)-thgrd(jp1,i,l))
ctdf specific humidity
                shjp1=shgrd(jp1,i,l)
     +              +xscale*(shgrd(jp1,ip1,l)-shgrd(jp1,i,l))
                 djp1=qdfgrd(jp1,i,l)
     +              +xscale*(qdfgrd(jp1,ip1,l)-qdfgrd(jp1,i,l))
c altitude                                    tdf 9/9/2K5
                 altjp1=altgrd(jp1,i,l)
     +              +xscale*(altgrd(jp1,ip1,l)-qdfgrd(jp1,i,l))
c level lp1
                pvj2=pvgrd(j,i,lp1)
     +              +xscale*(pvgrd(j,ip1,lp1)-pvgrd(j,i,lp1))
                 wj2=wgrd(j,i,lp1)
     +              +xscale*(wgrd(j,ip1,lp1)-wgrd(j,i,lp1))
              xmsfj2=xmsfgrd(j,i,lp1)
     +              +xscale*(xmsfgrd(j,ip1,lp1)-xmsfgrd(j,i,lp1))
ctdf potential temperature
                 thj2=thgrd(j,i,lp1)
     +              +xscale*(thgrd(j,ip1,lp1)-thgrd(j,i,lp1))
ctdf specific humidity
                 shj2=shgrd(j,i,lp1)
     +              +xscale*(shgrd(j,ip1,lp1)-shgrd(j,i,lp1))
ctdf
                  dj2=qdfgrd(j,i,lp1)
     +              +xscale*(qdfgrd(j,ip1,lp1)-qdfgrd(j,i,lp1))
c altitude                                    tdf 9/9/2K5
                  altj2=altgrd(j,i,lp1)
     +              +xscale*(altgrd(j,ip1,lp1)-altgrd(j,i,lp1))
c
               pvjp2=pvgrd(jp1,i,lp1)+
     +              +xscale*(pvgrd(jp1,ip1,lp1)-pvgrd(jp1,i,lp1))
                wjp2=wgrd(jp1,i,lp1)
     +              +xscale*(wgrd(jp1,ip1,lp1)-wgrd(jp1,i,lp1))
             xmsfjp2=xmsfgrd(jp1,i,lp1)
     +              +xscale*(xmsfgrd(jp1,ip1,lp1)-xmsfgrd(jp1,i,lp1))
ctdf potential temperature
                thjp2=thgrd(jp1,i,lp1)
     +              +xscale*(thgrd(jp1,ip1,lp1)-thgrd(jp1,i,lp1))
ctdf specific humidity
                shjp2=shgrd(jp1,i,lp1)
     +              +xscale*(shgrd(jp1,ip1,lp1)-shgrd(jp1,i,lp1))
ctdf
                 djp2=qdfgrd(jp1,i,lp1)
     +              +xscale*(qdfgrd(jp1,ip1,lp1)-qdfgrd(jp1,i,lp1))
c altitude                                    tdf 9/9/2K5
                 altjp2=altgrd(jp1,i,lp1)
     +              +xscale*(altgrd(jp1,ip1,lp1)-altgrd(jp1,i,lp1))
c level l
                 pv1=pvj1+yscale*(pvjp1-pvj1)
               xmsf1=xmsfj1+yscale*(xmsfjp1-xmsfj1)
ctdf potential temperature
                 th1=thj1+yscale*(thjp1-thj1)
ctdf specific humidity
                  sh1=shj1+yscale*(shjp1-shj1)
ctdf
                   d1=dj1+yscale*(djp1-dj1)
c altitude                                    tdf 9/9/2K5
                   alt1=altj1+yscale*(altjp1-altj1)
c level lp1
                 pv2=pvj2+yscale*(pvjp2-pvj2)
               xmsf2=xmsfj2+yscale*(xmsfjp2-xmsfj2)
ctdf specific humidity
                  sh2=shj2+yscale*(shjp2-shj2)
ctdf potential temperature
                  th2=thj2+yscale*(thjp2-thj2)
                   d2=dj2+yscale*(djp2-dj2)
c altitude                                    tdf 9/9/2K5
                   alt2=altj2+yscale*(altjp2-altj2)
ctdf
                pvn(n)=pv1+zscale*(pv2-pv1)
              xmsfn(n)=xmsf1+zscale*(xmsf2-xmsf1)
ctdf specific humidity
                shn(n)=sh1+zscale*(sh2-sh1)
ctdf potential temperature
                thn(n)=th1+zscale*(th2-th1)
ctdf strain rotation
               qdfn(n)=d1+zscale*(d2-d1)
ctdf diagnose eta                   tdf 7/28/2K5
                 zn(n)= z1+zscale*(z2-z1)
c altitude                                    tdf 9/9/2K5
                 altn(n)= alt1+zscale*(alt2-alt1)
c
                goto 45 
                endif
              end do   ! end of loop over levels
            endif
   30     continue     ! end of loop over longitude
          endif
   40   continue       ! end of loop over latitude
   45   continue
c update minimum temperature                            Duncan 7/18/2000
        rkap=0.286 
ctdf replace zn (now eta) with thn (potential temperature)  tdf 11/05/2K4
ctdf        tn = zn(n)*(pn(n)/1000.)**rkap 
        tn = thn(n)*(pn(n)/1000.)**rkap 
ctdf        if (tn .lt. xmint(n)) xmint(n) = tn 
        if (n .le. 50) then 
c          print *,' getdiag'
c          print *,' n,xn,yn,pn,altn,pvn,xmsfn,thn,shn,tn,zn'
c          print *, n,xn(n),yn(n),pn(n),altn(n),
c     1             pvn(n),xmsfn(n),thn(n),shn(n),tn,zn(n)
        endif
   50 continue
      return
      end  


      subroutine getdiag_calinit(ist,last)
c
c*****************************************************************c
c                                                                 c
c   Diagnose PV, theta, p, sp. hum., and QDF                      c
c   for each air parcel                                           c
c   INPUT: xn, yn, altn, ps (diagnosed)                           c
c   OUTPUT: pvn, thn, pn, shn, qdfn, zn                           c
c                                                                 c
c*****************************************************************c
c
      include 'comm_gfs_3d_v01.'
      real xmsfj1,xmsfjp1,xmsfj2,xmsfjp2,xmsf1,xmsf2
      real pw(nth), zw(nth), altw(nth)
      real lpnmc(nth)
      lpnmc(1)=      10.0000
      lpnmc(2)=      20.0000
      lpnmc(3)=      30.0000
      lpnmc(4)=      50.0000
      lpnmc(5)=      70.0000
      lpnmc(6)=      100.000
      lpnmc(7)=      150.000
      lpnmc(8)=      200.000
      lpnmc(9)=      250.000
      lpnmc(10)=     300.000
      lpnmc(11)=     350.000
      lpnmc(12)=     400.000
      lpnmc(13)=     450.000
      lpnmc(14)=     500.000
      lpnmc(15)=     550.000
      lpnmc(16)=     600.000
      lpnmc(17)=     650.000
      lpnmc(18)=     700.000
      lpnmc(19)=     750.000
      lpnmc(20)=     800.000
      lpnmc(21)=     850.000
      lpnmc(22)=     900.000
      lpnmc(23)=     925.000
      lpnmc(24)=     950.000
      lpnmc(25)=     975.000
      lpnmc(26)=     1000.00

      nthw = nth
      do 50 n = ist, last
        xp=xn(n)
        yp=yn(n)
ctdf geopotential altitude 
        altp=altn(n)
        if(xp.lt.x(1))xp=xp+360.

c First, need to establish xscale and yscale

          do 40 i=1,nc
            ip1=i+1
            if(i.eq.nc)ip1=1
            xlon=x(i) 
            xlonp1=x(ip1)
            if(i.eq.nc)xlonp1=360.+x(ip1)
c VLH 5/2/02            if((xp.ge.xlon).and.(xp.lt.xlonp1))then
            if((xp.ge.xlon).and.(xp.le.xlonp1))then
            xscale=(xp-xlon)/(xlonp1-xlon)
            do 30 j=1,nr-1
              jp1=j+1
              xlat=y(j)
              xlatp1=y(jp1)
c
c note: does not interpolate across the poles	VLH 5/8/02
c
              if((yp.ge.xlat).and.(yp.le.xlatp1))then
c DF 10/16/97           if((yp.ge.xlat).and.(yp.lt.xlatp1))then
                yscale=(yp-xlat)/(xlatp1-xlat)

ctdf Now have xscale, yscale for chosen parcel, n
ctdf Find surface pressure at chosen location
                psj=psfcgrd(j,i)+
     +              xscale*(psfcgrd(j,ip1)-psfcgrd(j,i))
                psjp=psfcgrd(jp1,i)+
     +              xscale*(psfcgrd(jp1,ip1)-psfcgrd(jp1,i))
                ps=psj+yscale*(psjp-psj)

ctdf Now construct pressure and eta profiles at location
                do l = 1, nthw
                  pw(l) = lpnmc(l)
                  zw(l) = lpnmc(l)
ctdf geopotential altitude profile
                  asj = altgrd(j,i,l)+
     +              xscale*(altgrd(j,ip1,l)-altgrd(j,i,l))
                  asjp = altgrd(jp1,i,l)+
     +              xscale*(altgrd(jp1,ip1,l)-altgrd(jp1,i,l))
ctdf geopotential altitude profile
                  altw(l)=asj+yscale*(asjp-asj)
                end do
ctdf Pressure profile constructed at location
                do l = 1, nthw-1
                  lp1 = l+1
c pressure
                  pl = pw(l)
                  plp1 = pw(lp1)
c geop. altitude
                  altl = altw(l)
                  altlp1 = altw(lp1)
c eta            
                  z1 = zw(l)
                  z2 = zw(lp1)
c
ctdf NB top to bottom ordering means that pw(lp1)>pw(l)
ctdf                  if((pp.le.pl).and.(pp.ge.plp1))then
ctdf NB top to bottom ordering means that altw(lp1)<altw(l)
                  if((altp.le.altl).and.(altp.ge.altlp1))then
ctdf linear in altitude
                    zscale=(altp-altl)/(altlp1-altl)
ctdf diagnose pressure                        tdf 9/09/2K5
                    pp = pl + zscale*(plp1-pl)
c      print *,' getdiag_calinit'
c      print *,'altl,altp,altlp1,pl,pp,plp1,zscale'
c      print *,altl,altp,altlp1,pl,pp,plp1,zscale
ctdf
ctdf linear in p    zscale=(pp-pl)/(plp1-pl)
ctdf linear log(p)  zscale=alog(pp/pl)/alog(plp1/pl)
ctdf
ctdf Now have xscale, yscale, zscale for chosen parcel, n
c level l
                pvj1=pvgrd(j,i,l)+xscale*(pvgrd(j,ip1,l)-pvgrd(j,i,l))
                 wj1=wgrd(j,i,l)+xscale*(wgrd(j,ip1,l)-wgrd(j,i,l))
              xmsfj1=xmsfgrd(j,i,l)
     +              +xscale*(xmsfgrd(j,ip1,l)-xmsfgrd(j,i,l))
ctdf potential temperature
              thj1=thgrd(j,i,l)
     +              +xscale*(thgrd(j,ip1,l)-thgrd(j,i,l))
ctdf specific humidity
              shj1=shgrd(j,i,l)
     +              +xscale*(shgrd(j,ip1,l)-shgrd(j,i,l))
c QDF
                 dj1=qdfgrd(j,i,l)
     +              +xscale*(qdfgrd(j,ip1,l)-qdfgrd(j,i,l))

               pvjp1=pvgrd(jp1,i,l)
     +              +xscale*(pvgrd(jp1,ip1,l)-pvgrd(jp1,i,l))
             xmsfjp1=xmsfgrd(jp1,i,l)
     +              +xscale*(xmsfgrd(jp1,ip1,l)-xmsfgrd(jp1,i,l))
                wjp1=wgrd(jp1,i,l)
     +              +xscale*(wgrd(jp1,ip1,l)-wgrd(jp1,i,l))
ctdf potential temperature
                thjp1=thgrd(jp1,i,l)
     +              +xscale*(thgrd(jp1,ip1,l)-thgrd(jp1,i,l))
ctdf specific humidity
                shjp1=shgrd(jp1,i,l)
     +              +xscale*(shgrd(jp1,ip1,l)-shgrd(jp1,i,l))
                 djp1=qdfgrd(jp1,i,l)
     +              +xscale*(qdfgrd(jp1,ip1,l)-qdfgrd(jp1,i,l))
c level lp1
                pvj2=pvgrd(j,i,lp1)
     +              +xscale*(pvgrd(j,ip1,lp1)-pvgrd(j,i,lp1))
              xmsfj2=xmsfgrd(j,i,lp1)
     +              +xscale*(xmsfgrd(j,ip1,lp1)-xmsfgrd(j,i,lp1))
                 wj2=wgrd(j,i,lp1)
     +              +xscale*(wgrd(j,ip1,lp1)-wgrd(j,i,lp1))
ctdf potential temperature
                 thj2=thgrd(j,i,lp1)
     +              +xscale*(thgrd(j,ip1,lp1)-thgrd(j,i,lp1))
ctdf specific humidity
                 shj2=shgrd(j,i,lp1)
     +              +xscale*(shgrd(j,ip1,lp1)-shgrd(j,i,lp1))
ctdf
                  dj2=qdfgrd(j,i,lp1)
     +              +xscale*(qdfgrd(j,ip1,lp1)-qdfgrd(j,i,lp1))
c
               pvjp2=pvgrd(jp1,i,lp1)+
     +              +xscale*(pvgrd(jp1,ip1,lp1)-pvgrd(jp1,i,lp1))
             xmsfjp2=xmsfgrd(jp1,i,lp1)
     +              +xscale*(xmsfgrd(jp1,ip1,lp1)-xmsfgrd(jp1,i,lp1))
                wjp2=wgrd(jp1,i,lp1)
     +              +xscale*(wgrd(jp1,ip1,lp1)-wgrd(jp1,i,lp1))
ctdf potential temperature
                thjp2=thgrd(jp1,i,lp1)
     +              +xscale*(thgrd(jp1,ip1,lp1)-thgrd(jp1,i,lp1))
ctdf specific humidity
                shjp2=shgrd(jp1,i,lp1)
     +              +xscale*(shgrd(jp1,ip1,lp1)-shgrd(jp1,i,lp1))
ctdf
                 djp2=qdfgrd(jp1,i,lp1)
     +              +xscale*(qdfgrd(jp1,ip1,lp1)-qdfgrd(jp1,i,lp1))
c level l
                 pv1=pvj1+yscale*(pvjp1-pvj1)
               xmsf1=xmsfj1+yscale*(xmsfjp1-xmsfj1)
ctdf potential temperature
                 th1=thj1+yscale*(thjp1-thj1)
ctdf specific humidity
                  sh1=shj1+yscale*(shjp1-shj1)
ctdf
                   d1=dj1+yscale*(djp1-dj1)
c level lp1
                 pv2=pvj2+yscale*(pvjp2-pvj2)
               xmsf2=xmsfj2+yscale*(xmsfjp2-xmsfj2)
ctdf specific humidity
                  sh2=shj2+yscale*(shjp2-shj2)
ctdf potential temperature
                  th2=thj2+yscale*(thjp2-thj2)
                   d2=dj2+yscale*(djp2-dj2)
ctdf
                pvn(n)=pv1+zscale*(pv2-pv1)
              xmsfn(n)=xmsf1+zscale*(xmsf2-xmsf1)
ctdf specific humidity
                shn(n)=sh1+zscale*(sh2-sh1)
ctdf potential temperature
                thn(n)=th1+zscale*(th2-th1)
ctdf strain rotation
               qdfn(n)=d1+zscale*(d2-d1)
ctdf diagnose eta                   tdf 7/28/2K5
                 zn(n)= z1+zscale*(z2-z1)
ctdf add in p                       tdf 9/09/2K5
                 pn(n) = pp
               altn(n) = altp
c
                goto 45 
                endif
              end do   ! end of loop over levels
            endif
   30     continue     ! end of loop over longitude
          endif
   40   continue       ! end of loop over latitude
   45   continue
c update minimum temperature                            Duncan 7/18/2000
        rkap=0.286 
ctdf replace zn (now eta) with thn (potential temperature)  tdf 11/05/2K4
ctdf        tn = zn(n)*(pn(n)/1000.)**rkap 
        tn = thn(n)*(pn(n)/1000.)**rkap 
ctdf        if (tn .lt. xmint(n)) xmint(n) = tn 
        if (n .le. 50) then 
c          print *,' getdiag_calinit'
c          print *,' n,xn,yn,pn,altn,pvn,xmsfn,thn,shn,tn,zn'
c          print *, n,xn(n),yn(n),pn(n),altn(n),
c     1             pvn(n),xmsfn(n),thn(n),shn(n),tn,zn(n)
        endif
   50 continue
      return
      end  



      subroutine calendar (istime,etime,ictime) 
c*****************************************************************c
c  Compute the current time, ictime, from the start time, istime, c
c  and the elapsed time in hours, etime.   Both ictime and istime c
c  are integer*4 and expressed in the form "yymmddhh"             c
c                                                                 c
c  Simulations less than 2 years are supported                    c
c                                                                 c
c  This version accomodates back trajectories (VLH 1/28/98)       c
c  Corrected for 12Z winds situation          (VLH 7/01/99)       c
c*****************************************************************c

      integer*4 istime,ictime,yy1,mm1,dd1,hh1,yy2,mm2,dd2,hh2
      integer mno(12)
      real etime
      data (mno(i),i=1,12)/31,28,31,30,31,30,31,31,30,31,30,31/

      yy1=istime/1000000
      if (mod(yy1,4) .eq. 0.) mno(2)=29
      yy2=yy1
      mm1=istime/10000-yy1*100
      mm2=mm1
      dd1=istime/100-yy1*10000-mm1*100
c
c --- New day                                   Duncan 5/11/2000
C      dd2=dd1+nint(etime)/24
      dd2=dd1+ifunc(etime)/24
      hh1=istime-yy1*1000000-mm1*10000-dd1*100

c --- Forward trajectories
      if (etime .gt. 0.) then
10        if (dd2 .gt. mno(mm1)) then
              dd2=dd2-mno(mm1)
              mm2=mm1+1
              if (mm2 .gt. 12) then
                  mm2=mm2-12
                  yy2=yy1+1
                  if (mod(yy2,4).eq.0.) mno(2)=29
                  if (mod(yy2,4).ne.0.) mno(2)=28
              endif
          endif
          do j=2,12
             if (dd2 .gt. mno(mm2)) then
                 dd2=dd2-mno(mm2)
                 mm2=mm1+j
                 if (mm2 .gt. 12) then
                     mm2=mm2-12
                     yy2=yy1+1
                     if (mod(yy2,4).eq.0.) mno(2)=29
                     if (mod(yy2,4).ne.0.) mno(2)=28
                 endif
             endif
          enddo
          if (dd2 .gt. mno(mm2)) goto 10

c --- Back trajectories
      elseif (etime .lt. 0.) then
20        if (dd2 .le. 0.) then
              mm2=mm1-1
              if (mm2 .eq. 0) then
                  mm2=mm2+12
                  yy2=yy1-1
                  if (mod(yy2,4).eq.0.) mno(2)=29
                  if (mod(yy2,4).ne.0.) mno(2)=28
              endif
              dd2=dd2+mno(mm2)
          endif
          do j=2,12
             if (dd2 .le. 0.) then
                 mm2=mm1-j
                 if (mm2 .le. 0) then
                     mm2=mm2+12
                     yy2=yy1-1
                     if (mod(yy2,4).eq.0.) mno(2)=29
                     if (mod(yy2,4).ne.0.) mno(2)=28
                 endif
                 dd2=dd2+mno(mm2)
             endif
          enddo
          if (dd2 .le. 0.) goto 20
      endif

c --- New hour                                   Duncan 5/11/2000
C      hh2=mod(nint(etime),24)+hh1
      hh2=mod(ifunc(etime),24)+hh1
      if (etime.gt.0.) then
ctdf         if (hh2.le. 0) hh2=hh2+24  error       tdf 9/9/2K5
         if (hh2.lt. 0) hh2=hh2+24
         if (hh2.ge.24) then
            hh2=hh2-24
            dd2=dd2+1
            if (dd2.gt.mno(mm2)) then
               dd2 = dd2 - mno(mm2)
               mm2=mm2+1
               if (mm2.gt.12) then
                  mm2=mm2-12
                  yy2=yy2+1
               endif
            endif
         endif
      else if (etime.lt.0.) then
         if (hh2.lt. 0) then
            hh2=hh2+24
            dd2=dd2-1
            if (dd2.lt.1) then
               mm2=mm2-1
               if (mm2.lt.1) mm2=12
               dd2=mno(mm2)
               if (mm2.eq.12) yy2=yy2-1
            endif
         endif
      endif

c --- Current time
      ictime=hh2+dd2*100+mm2*10000+yy2*1000000

      return
      end
      function ifunc(etime)
c Function subroutine that returns the integer immediately
c below the real number etime          Duncan 4/21/2000 
      if (etime .ge. 0.0) then
        ifunc = ifix(etime)
      endif
      if (etime .lt. 0.0) then
        ifunc = ifix(etime) 
        del = amod(etime,1.) 
        if (del .lt. 0.0) ifunc = ifunc-1 
      endif
      ifunc = ifunc
      return
      end 
c****************************************************************

      subroutine traject
c
c*****************************************************************c
c                                                                 c
c  Timestep trajectories using fourth-order Runge-Kutta scheme    c
c                                                                 c
c*****************************************************************c
c
      include 'comm_gfs_3d_v01.'
      real m1,n1,l1
      real m2,n2,l2
      real m3,n3,l3
      real m4,n4,l4
      PI2 = 6.2831853071796
      RAD=6.37E6
      radg=pi2/360.
      pi=.5*pi2
c
      do 100 n=1,ntraj
c
ctdf Here, w is p-dot
        call interp_gfs(u,v,w,uigw,vigw,xn(n),yn(n),pn(n),ps)
c
        m1=dt*(180./pi)*u/(rad*cos(radg*yn(n)))
        n1=dt*(180./pi)*v/rad
        l1=dt*w
        xprm = xn(n) + .5*m1
        yprm = yn(n) + .5*n1
        pprm = pn(n) + .5*l1
c
        call interp_gfs(u,v,w,uigw,vigw,xprm,yprm,pprm,ps)
c
        m2=dt*(180./pi)*u/(rad*cos(radg*yprm))
        n2=dt*(180./pi)*v/rad
        l2=dt*w
        xprm = xn(n) + .5*m2
        yprm = yn(n) + .5*n2
        pprm = pn(n) + .5*l2
c
        call interp_gfs(u,v,w,uigw,vigw,xprm,yprm,pprm,ps)
c
        m3=dt*(180./pi)*u/(rad*cos(radg*yprm))
        n3=dt*(180./pi)*v/rad
        l3=dt*w
        xprm = xn(n) + m3
        yprm = yn(n) + n3
        pprm = pn(n) + l3
c
        call interp_gfs(u,v,w,uigw,vigw,xprm,yprm,pprm,ps)
c
        m4=dt*(180./pi)*u/(rad*cos(radg*yprm))
        n4=dt*(180./pi)*v/rad
        l4=dt*w
        xn(n) = xn(n) + (m1+2.*m2+2.*m3+m4)/6.
        yn(n) = yn(n) + (n1+2.*n2+2.*n3+n4)/6.
c Update pn
        pn(n) = pn(n) + (l1+2.*l2+2.*l3+l4)/6.
        if(pn(n).gt.ps)pn(n)=ps
c Adding agen variable                                Duncan 11/05/99 
        agen(n) = agen(n) + dt/3600.
c
c  adjust for polar points
c
        itest=0
50      continue
        if (yn(n).lt.-90.) xn(n)= 180.+xn(n)
        if (yn(n).lt.-90.) yn(n)= -180-yn(n)
        if (yn(n).gt.90.)  xn(n)= 180.+xn(n)
        if (yn(n).gt.90.)  yn(n)= 180-yn(n)
        if (xn(n).lt.0.) xn(n)= 360.+xn(n)
        if (xn(n).ge.360.) xn(n)= -360.+xn(n)
        itest=itest+1
c   if(itest.le.3)goto 50
100   continue
      return
      end

      subroutine isopleth(ciso,ds)
c
c*****************************************************************c
c                                                                 c
c   Initialises parcels on PV isopleths or isotherms              c
c   or streamlines (version 12 update)                            c 
c                                                                 c
c*****************************************************************c
c
      include 'comm_gfs_3d_v01.'
      print *,'isopleth routine called'
      print *,' tiso=',tiso,' pviso=',pviso,' stream=',stream 
c      print *,'nc=',nc,' nr=',nr
      DX=DLON
      nthw = nth 
      do 30 l = 1, nthw
      theta =thw(l)
      print *,'thw(l)=',theta
      DO 20 I=2,NC
        IM1=I-1
        DO 20 J=1,NR
          JM1=J-1
          IF(J.EQ.1)JM1=1
          HY=Y(J)-Y(JM1)
          if(pviso) then
c tdf NB Changed ordering of "sim" arrays        tdf 02/04/2K5
            A1=(PVsim(IM1,JM1,L))
            B1=(PVsim(IM1,J,L))
            C1=(PVsim(I,JM1,L))
            D1=(PVsim(I,J,L))
          endif
          if(tiso) then
            A1=theta*(Psim(IM1,JM1,L)/1000.)**.286
            B1=theta*(Psim(IM1,J,L)/1000.)**.286
            C1=theta*(Psim(I,JM1,L)/1000.)**.286
            D1=theta*(Psim(I,J,L)/1000.)**.286
          endif
          if(stream) then
            A1= xmsfsim(IM1,JM1,L)
            B1= xmsfsim(IM1,J,L)
            C1= xmsfsim(I,JM1,L)
            D1= xmsfsim(I,J,L)
c tdf NB Changed ordering of "sim" arrays        tdf 02/04/2K5
          endif
C
C  LOWER LEFT TRIANGLE EMPTY 
C
          IF((A1.GE.CISO).AND.(B1.GE.CISO).AND.
     &       (C1.LT.CISO).AND.(D1.GE.CISO))THEN
            A=DX*(C1-CISO)/(C1-D1)
            B=HY*(C1-CISO)/(C1-A1)
            II=I
            JJ=J
            CALL FILL(ds,A,B,A1,B1,C1,D1,II,JJ,IM1,JM1,1,theta)
            GO TO 10
          ENDIF 
C
C  LOWER LEFT TRIANGLE FULL 
C
          IF((A1.LT.CISO).AND.(B1.LT.CISO).AND.
     &       (C1.GE.CISO).AND.(D1.LT.CISO))THEN
            A=DX*(C1-CISO)/(C1-D1)
            B=HY*(C1-CISO)/(C1-A1)
            II=I
            JJ=J
            CALL FILL(DS,A,B,A1,B1,C1,D1,II,JJ,IM1,JM1,2,theta)
            GO TO 10
          ENDIF 
C
C  UPPER LEFT TRIANGLE EMPTY 
C
          IF((A1.LT.CISO).AND.(B1.GE.CISO).AND.
     &       (C1.GE.CISO).AND.(D1.GE.CISO))THEN
            A=DX*(A1-CISO)/(A1-B1)
            B=HY*(A1-CISO)/(A1-C1)
            II=I
            JJ=J
            CALL FILL(DS,A,B,A1,B1,C1,D1,II,JJ,IM1,JM1,3,theta)
            GO TO 10
          ENDIF 
C
C  UPPER LEFT TRIANGLE FULL 
C
          IF((A1.GE.CISO).AND.(B1.LT.CISO).AND.
     &       (C1.LT.CISO).AND.(D1.LT.CISO))THEN
            A=DX*(A1-CISO)/(A1-B1)
            B=HY*(A1-CISO)/(A1-C1)
            II=I
            JJ=J
            CALL FILL(DS,A,B,A1,B1,C1,D1,II,JJ,IM1,JM1,4,theta)
            GO TO 10
          ENDIF 
C
C  UPPER RIGHT TRIANGLE EMPTY 
C
          IF((A1.GE.CISO).AND.(B1.LT.CISO).AND.
     &       (C1.GE.CISO).AND.(D1.GE.CISO))THEN
            A=DX*(B1-CISO)/(B1-A1)
            B=HY*(B1-CISO)/(B1-D1)
            II=I
            JJ=J
            CALL FILL(DS,A,B,A1,B1,C1,D1,II,JJ,IM1,JM1,5,theta)
            GO TO 10
          ENDIF 
C
C  UPPER RIGHT TRIANGLE FULL 
C
          IF((A1.LT.CISO).AND.(B1.GE.CISO).AND.
     &       (C1.LT.CISO).AND.(D1.LT.CISO))THEN
            A=DX*(B1-CISO)/(B1-A1)
            B=HY*(B1-CISO)/(B1-D1)
            II=I
            JJ=J
            CALL FILL(DS,A,B,A1,B1,C1,D1,II,JJ,IM1,JM1,6,theta)
            GO TO 10
          ENDIF 
C
C  LOWER RIGHT TRIANGLE EMPTY 
C
          IF((A1.GE.CISO).AND.(B1.GE.CISO).AND.
     &       (C1.GE.CISO).AND.(D1.LT.CISO))THEN
            A=DX*(D1-CISO)/(D1-C1)
            B=HY*(D1-CISO)/(D1-B1)
            II=I
            JJ=J
            CALL FILL(DS,A,B,A1,B1,C1,D1,II,JJ,IM1,JM1,7,theta)
            GO TO 10
          ENDIF 
C
C  LOWER RIGHT TRIANGLE FULL
C
          IF((A1.LT.CISO).AND.(B1.LT.CISO).AND.
     &       (C1.LT.CISO).AND.(D1.GE.CISO))THEN
            A=DX*(D1-CISO)/(D1-C1)
            B=HY*(D1-CISO)/(D1-B1)
            II=I
            JJ=J
            CALL FILL(DS,A,B,A1,B1,C1,D1,II,JJ,IM1,JM1,8,theta)
            GO TO 10
          ENDIF 
C
C  RIGHT TRAPEZOID EMPTY
C
          IF((A1.GE.CISO).AND.(B1.LT.CISO).AND.
     &       (C1.GE.CISO).AND.(D1.LT.CISO))THEN
            A=DX*(A1-CISO)/(A1-B1)
            B=DX*(C1-CISO)/(C1-D1)
            II=I
            JJ=J
            CALL FILL(DS,A,B,A1,B1,C1,D1,II,JJ,IM1,JM1,9,theta)
            GO TO 10
          ENDIF 
C
C  RIGHT TRAPEZOID FULL
C
          IF((A1.LT.CISO).AND.(B1.GE.CISO).AND.
     &       (C1.LT.CISO).AND.(D1.GE.CISO))THEN
            A=DX*(A1-CISO)/(A1-B1)
            B=DX*(C1-CISO)/(C1-D1)
            II=I
            JJ=J
            CALL FILL(DS,A,B,A1,B1,C1,D1,II,JJ,IM1,JM1,10,theta)
            GO TO 10
          ENDIF 
C
C  LOWER TRAPEZOID EMPTY 
C
          IF((A1.GE.CISO).AND.(B1.GE.CISO).AND.
     &       (C1.LT.CISO).AND.(D1.LT.CISO))THEN
            A=HY*(C1-CISO)/(C1-A1)
            B=HY*(D1-CISO)/(D1-B1)
            II=I
            JJ=J
            CALL FILL(DS,A,B,A1,B1,C1,D1,II,JJ,IM1,JM1,11,theta)
            GO TO 10
          ENDIF 
C
C  LOWER TRAPEZOID FULL 
C
          IF((A1.LT.CISO).AND.(B1.LT.CISO).AND.
     &       (C1.GE.CISO).AND.(D1.GE.CISO))THEN
            A=HY*(C1-CISO)/(C1-A1)
            B=HY*(D1-CISO)/(D1-B1)
            II=I
            JJ=J
            CALL FILL(DS,A,B,A1,B1,C1,D1,II,JJ,IM1,JM1,12,theta)
            GO TO 10
          ENDIF 
10      CONTINUE
20    CONTINUE
30    continue
      return
      end
      subroutine spacefill
c
c*****************************************************************c
c                                                                 c
c   Fill space with air parcels                                   c
c                                                                 c
c*****************************************************************c
c
      include 'comm_gfs_3d_v01.'
      n=ntraj
      write(6,*),' ntraj =',ntraj,' entering spacefill'
       dx=.1
       dy=.1
c
c wdnr
c 
       x0=-95.
       y0=40.
c
c beijing
c
       x0=109.
       y0=35.

       do i=1,150
        do j=1,100
         do l=1,4
             n=n+1
             xn(n)=x0+(i-1)*dx
             yn(n)=y0+(j-1)*dy
             pn(n)=1000.
             if(n.ge.nmax)return
         enddo
        enddo
       enddo
      ntraj=n
        do n=1,ntraj,4
        xp=xn(n)
        yp=yn(n)
          do i=1,ncmax
            ip1=i+1
            if(i.eq.ncmax)ip1=1
            xlon=x(i)
            xlonp1=x(ip1)
            if(i.eq.ncmax)xlonp1=360.+x(ip1)
            if((xp.ge.xlon).and.(xp.le.xlonp1))then
            xscale=(xp-xlon)/(xlonp1-xlon)
            do j=1,nrmax-1
              jp1=j+1
              xlat=y(j)
              xlatp1=y(jp1)
              if((yp.ge.xlat).and.(yp.le.xlatp1))then
                yscale=(yp-xlat)/(xlatp1-xlat)
                psj=psfcgrd(j,i)+
     +              xscale*(psfcgrd(j,ip1)-psfcgrd(j,i))
                psjp=psfcgrd(jp1,i)+
     +              xscale*(psfcgrd(jp1,ip1)-psfcgrd(jp1,i))
                ps=psj+yscale*(psjp-psj)
                pn(n)=ps-50.
                pn(n+1)=ps-100.
                pn(n+2)=ps-150.
                pn(n+3)=ps-200.
              endif
            enddo 
            endif
         enddo
         enddo
999   continue
      write(6,*),' ntraj =',ntraj,' exiting spacefill'
      return
      END

c      subroutine spacefill
c
c*****************************************************************c
c                                                                 c
c   Fill space with air parcels                                   c
c                                                                 c
c*****************************************************************c
c
c      include 'comm_gfs_3d_v01.'
c      n=ntraj
c      write(6,*),' ntraj =',ntraj,' entering spacefill'
c          do i=1,ncmax,2
c            xlon=x(i)
c            do j=1,nrmax,2
c              xlat=y(j)
c
c initialize trajectories in 50mb increments from surface
c
c              do l=1, 4
c                 n=n+1
c                 xn(n)=xlon
c                 yn(n)=xlat
ctdf initialise pn, not zn                         tdf 7/29/2K5
c                 pn(n)=psfcgrd(j,i)-50.*l
c
c incrementing n after assigning xn,yn,zn results in 1 extra traj
c
c                 if(n.ge.nmax)return
c              enddo
c            enddo
c          enddo
c      ntraj=n
c      write(6,*),' ntraj =',ntraj,' exiting spacefill'
c      return
c      END

      subroutine wildfire_fill
c
c*****************************************************************c
c                                                                 c
c   Fill space with wildfire air parcels                          c
c                                                                 c
c*****************************************************************c
c
      include 'comm_gfs_3d_v01.'
      write(6,*),' ntraj =',ntraj,' entering wildfire_fill'
c
c open active wildfire site file
c
      OPEN(94,FILE='wild_fire_trajectories.dat',
     &status='old',READONLY)
      firemax=0
      read(94,*)firemax
      x0=0.
      y0=0.
      z0=0.
      do nf=1,firemax
      read(94,*)x0,y0,z0
      print *,x0,y0,z0
             n=n+1
             xn(n)=x0
             yn(n)=y0
             pn(n)=z0
             if(n.ge.nmax)return
      enddo
      ntraj=n
      write(6,*),' ntraj =',ntraj,' exiting wildfire_fill'
      close(94)
      return
      END

      subroutine er2_range_fill
c-----------------------------------------------------------------
c   Call range_ring to fill space in the range of the ER2
c-----------------------------------------------------------------
c
c INPUTS from common_ashoe_v08:   
c         stlat   = center latitude (in degs.)
c         stlon   = center longitude (in degs.)
c         er2_range  = ER2 range from the central point (in KM)
c         npts    = number of points in each ring
c                   (if zero, defaults to 36)
c         nrings = number of rings
c
      include 'comm_gfs_3d_v01.'
      parameter (nprmax=36)
      real range,bearing(nprmax),xlatp(nprmax),xlonp(nprmax)
c
      n=ntraj
      write(6,*),' ntraj =',ntraj,' entering er2_range_fill'
      if (n .eq. 0) n = 1
      write(6,*)'er2 range fill'
      write(6,*)stlon,stlat,er2_range,nrings,npts
      do l = 1, nthp
        xn(n) = stlon
        yn(n) = stlat
        zn(n) = thp(l)
        n=n+1
      end do
      do nring = 1, nrings
        range = nring * er2_range / nrings
        call range_ring(range,bearing,xlatp,xlonp)
        do npt = 1, npts
          do l = 1, nthp
            xn(n)=xlonp(npt)
            yn(n)=xlatp(npt)
            zn(n) = thp(l)
            n=n+1
            if(n.ge.nmax)return
          end do
        end do
      end do
      ntraj=n
      write(6,*)' ntraj =',ntraj,' exiting er2_range_fill'
      return
      end

      subroutine range_ring(range,bearing,latp,lonp)
c
c   Routine based on range_ring.pro, by Paul Newman 93/02/25
c   Calculates the array of latitudes and longitudes
c   that are at a particular range from a central latitude
c   longitude site (forming a crcle around that point)
c INPUTS:   
c         stlat   = center latitude (in degs.)
c         stlon   = center longitude (in degs.)
c         range   = distance of the circle from the central point (in KM)
c         npts    = number of points in the latitude, longitude, distance
c                      arrays to be returned (if zero, defaults to 36)
c OUTPUTS:  
c         bearing = direction of point from the central point (0=N, 180=S, 
c                                                            270=W, 90=E)
c         latp    = an npts element vector of latitudes spaced 
c                   at del km increments
c         lonp    = corresponding longitudes for latp
c
      include 'comm_gfs_3d_v01.'
      parameter (nprmax=36)
      real bearing(nprmax),latp(nprmax),lonp(nprmax)
      real sinBB(nprmax),cosBB(nprmax),sinAA(nprmax),cosAA(nprmax)
      real sinCC(nprmax),cosCC(nprmax)
      real b(nprmax),bb(nprmax),sinb(nprmax),cosb(nprmax)
      real lons
c
      pi=3.141592
c ** radius of the earth in radians
      re=20000./pi
      rad=180./pi
c
      if (abs(stlat) .gt. 90) then 
        print *,'range_ring: check your central latitude',stlat
        return
      endif
c  central longitude      
      lons=abs(stlon)

      if (npts .eq. 0) npts=36

      start_bear = 0.
      ccw = 1.0
      do n = 1, npts
        bearing(n) = ccw * (n-1)*360./npts + start_bear
        bb(n) = bearing(n)/rad
      enddo
c
c ** c is great circle angle between st and our points

      if (stlat .ge. 90) then
         sina=1.0e-10
         cosa=sqrt(1.-sina**2)
      endif
      if (stlat .le. -90) then
         sina=1.0e-10
         cosa=-1.0*sqrt(1.-sina**2)
      endif
      if ((stlat .gt. -90) .and. (stlat .lt. 90)) then
         a=(90.-stlat)/rad
         sina=sin(a)
         cosa=cos(a)
      endif
c
      c=range/re

      if (range .ge. 20000.) then 
        print *,'range_ring: Your range is too big',range
        range=19999.
        c=range/re
      endif

      sinc=sin(c)
      cosc=cos(c)

      do n = 1, npts
         cosBB(n) = cos(BB(n))
         sinBB(n) = sin(BB(n))
         cosb(n) = cosa*cosc+sina*sinc*cos(BB(n))
         b(n) = acos(cosb(n))
         sinb(n) = sin(b(n))
         latp(n) = 90. - b(n)*rad
c
c ** sincu and coscu are the arrays of sines and cosines of the longitudes

         sinAA(n) = sina * sinBB(n) /sinb(n)
         cosAA(n) = (cosa-cosc*cosb(n)) /sinc /sinb(n)
         sinCC(n) = sinc * sinBB(n) /sinb(n)
         cosCC(n) = - cosAA(n) * cosBB(n) + sinAA(n) * sinBB(n) * cosc
  
c ** calculate the lats and lons

         lonp(n)= atan2(sinCC(n),cosCC(n)) * rad + lons
      enddo
      return
      end

      SUBROUTINE FILL(DS,A,B,A1,B1,C1,D1,I,J,IM1,JM1,IFLAG,theta)
c
c*****************************************************************c
c                                                                 c
c   Routine fills in isopleths with air parcels                   c
c                                                                 c
c*****************************************************************c
c
      include 'comm_gfs_3d_v01.'
      rad=6.37e3
      PI2 = 6.2831853071796
      dtr=pi2/360.
      A=ABS(A)
      B=ABS(B)
C
C  LOWER LEFT TRIANGLE 
C
       IF(IFLAG.EQ.1.OR.IFLAG.EQ.2) THEN
        X0=X(IM1)
        if (i .eq. 1) x0 = x0-360.
        Y0=Y(J)-B
        sx=pi2*rad*a*cos(y(j)*dtr)/360.
        sy=pi2*rad*b/360.
        s=sqrt(sx**2+sy**2)
        ntick=s/ds
        if(ntick.gt.0)then
        DX= A/ntick
        DY= B/ntick
C       WRITE(6,*)'  LOWER LEFT TRIANGLE, ntick, dx = ' ,ntick,dx 
        DO 1 JJ=1,NTICK
         NTRAJ=NTRAJ+1
         IF(NTRAJ.LE.nmax)THEN
          XN(NTRAJ)=X0
          YN(NTRAJ)=Y0
          ZN(NTRAJ)=theta
          X0=X0+DX
          Y0=Y0+DY
         ENDIF
1       CONTINUE
        endif
       RETURN
       ENDIF 
C
C  UPPER LEFT TRIANGLE 
C
       IF(IFLAG.EQ.3.OR.IFLAG.EQ.4) THEN
        X0=X(IM1)
        if (i .eq. 1) x0 = x0-360.
        Y0=Y(JM1)+B
        sx=pi2*rad*a*cos(y(jm1)*dtr)/360.
        sy=pi2*rad*b/360.
        s=sqrt(sx**2+sy**2)
        ntick=s/ds
        if(ntick.gt.0)then
        DX= a/ntick
        DY= b/ntick
C       WRITE(6,*)'  UPPER LEFT TRIANGLE, ntick, dx = ' ,ntick,dx 
        DO 2 JJ=1,NTICK
         NTRAJ=NTRAJ+1
         IF(NTRAJ.LE.NMAX)THEN
          XN(NTRAJ)=X0
          YN(NTRAJ)=Y0
          ZN(NTRAJ)=theta
          X0=X0+DX
          Y0=Y0-DY
         ENDIF
2       CONTINUE
        endif
       RETURN
       ENDIF 
C
C  UPPER RIGHT TRIANGLE 
C
       IF(IFLAG.EQ.5.OR.IFLAG.EQ.6) THEN
        X0=X(I)
        Y0=Y(JM1)+B
        sx=pi2*rad*a*cos(y(jm1)*dtr)/360.
        sy=pi2*rad*b/360.
        s=sqrt(sx**2+sy**2)
        ntick=s/ds
        if(ntick.gt.0)then
        DX= a/ntick
        DY= b/ntick
C       WRITE(6,*)'  UPPER RIGHT TRIANGLE, ntick, dx = ' ,ntick,dx 
        DO 3 JJ=1,NTICK
         NTRAJ=NTRAJ+1
         IF(NTRAJ.LE.NMAX)THEN
          XN(NTRAJ)=X0
          YN(NTRAJ)=Y0
          ZN(NTRAJ)=theta
          X0=X0-DX
          Y0=Y0-DY
         ENDIF
3       CONTINUE
        endif
       RETURN
       ENDIF 
C
C  LOWER RIGHT TRIANGLE 
C
       IF(IFLAG.EQ.7.OR.IFLAG.EQ.8) THEN
        X0=X(I)
        Y0=Y(J)-B
        sx=pi2*rad*a*cos(y(j)*dtr)/360.
        sy=pi2*rad*b/360.
        s=sqrt(sx**2+sy**2)
        ntick=s/ds
        if(ntick.gt.0)then
        DX= a/ntick
        DY= b/ntick
C       WRITE(6,*)'  LOWER RIGHT TRIANGLE, ntick, dx = ' ,ntick,dx 
        DO 4 JJ=1,NTICK
         NTRAJ=NTRAJ+1
         IF(NTRAJ.LE.NMAX)THEN
          XN(NTRAJ)=X0
          YN(NTRAJ)=Y0
          ZN(NTRAJ)=theta
          X0=X0-DX
          Y0=Y0+DY
         ENDIF
4       CONTINUE
        endif
       RETURN
       ENDIF 
C
C  RIGHT TRAPEZOID 
C
       IF(IFLAG.EQ.9.OR.IFLAG.EQ.10) THEN
        X0=X(IM1)+A
        if (i .eq. 1) x0 = x0-360.
        Y0=Y(JM1)
        sx=pi2*rad*(b-a)*cos(y(j)*dtr)/360.
cDD        sy=pi2*rad*(y(j)-y(jm1))/360.
        sy=pi2*rad*(y(j)-y0)/360.
        s=sqrt(sx**2+sy**2)
        ntick=s/ds
        if(ntick.gt.0)then
        DX= (b-a)/ntick
cDD        DY= (y(j)-y(jm1))/ntick
        DY= (y(j)-y0)/ntick
C       WRITE(6,*)'  RIGHT TRAPEZOID, ntick, dx = ' ,ntick,dx 
        DO 5 JJ=1,NTICK
         NTRAJ=NTRAJ+1
         IF(NTRAJ.LE.NMAX)THEN
          XN(NTRAJ)=X0
          YN(NTRAJ)=Y0
          ZN(NTRAJ)=theta
          X0=X0+DX
          Y0=Y0+DY
         ENDIF
5       CONTINUE
        endif
       RETURN
       ENDIF 
C
C  LOWER TRAPEZOID 
C
       IF(IFLAG.EQ.11.OR.IFLAG.EQ.12) THEN
        X0=X(IM1)
        if (i .eq. 1) x0 = x0-360.
        Y0=Y(J)-A
cDD        sx=pi2*rad*(x(i)-x(im1))*cos((y(j)-a)*dtr)/360.
        sx=pi2*rad*(x(i)-x0)*cos((y(j)-a)*dtr)/360.
        sy=pi2*rad*(a-b)/360.
        s=sqrt(sx**2+sy**2)
        ntick=s/ds
        if(ntick.gt.0)then
cDD        DX= (x(i)-x(im1))/ntick
        DX= (x(i)-x0)/ntick
        DY= (a-b)/ntick
C       WRITE(6,*)'  LOWER TRAPEZOID, ntick, dx = ' ,ntick,dx 
        DO 6 JJ=1,NTICK
         NTRAJ=NTRAJ+1
         IF(NTRAJ.LE.NMAX)THEN
          XN(NTRAJ)=X0
          YN(NTRAJ)=Y0
          ZN(NTRAJ)=theta
          X0=X0+DX
          Y0=Y0+DY
         ENDIF
6       CONTINUE
        endif
       RETURN
       ENDIF 
      RETURN
      END

      subroutine interp_gfs(u,v,w,uigw,vigw,xp,yp,pp,ps)
c
c*****************************************************************c
c                                                                 c
c   Interpolate the velocity fields ugrd,vgrd,wgrd to find        c
c   the velocity u,v,w for the point xp,yp,zp                     c
c                                           tdf 7/12/2K5          c
c                                                                 c
c*****************************************************************c
c
      include 'comm_gfs_3d_v01.'
      real pw(nth), zw(nth)
      real lpnmc(nth)
      lpnmc(1)=      10.0000
      lpnmc(2)=      20.0000
      lpnmc(3)=      30.0000
      lpnmc(4)=      50.0000
      lpnmc(5)=      70.0000
      lpnmc(6)=      100.000
      lpnmc(7)=      150.000
      lpnmc(8)=      200.000
      lpnmc(9)=      250.000
      lpnmc(10)=     300.000
      lpnmc(11)=     350.000
      lpnmc(12)=     400.000
      lpnmc(13)=     450.000
      lpnmc(14)=     500.000
      lpnmc(15)=     550.000
      lpnmc(16)=     600.000
      lpnmc(17)=     650.000
      lpnmc(18)=     700.000
      lpnmc(19)=     750.000
      lpnmc(20)=     800.000
      lpnmc(21)=     850.000
      lpnmc(22)=     900.000
      lpnmc(23)=     925.000
      lpnmc(24)=     950.000
      lpnmc(25)=     975.000
      lpnmc(26)=     1000.00

c
      uigw=0.
      vigw=0.
      u=0.
      v=0.
      w=0.
      xpold=xp
      ypold=yp
      ppold=pp
ctdf
ctdf NB wind fields need to be ordered 0 -> 360
ctdf      print *,' interp_g4 called with xp,yp,pp ',xp,yp,pp
      if(xp.lt.x(1))xp=xp+360.
      nlw = nth 
c
c pole points
c
      lsp=0
      lnp=0
      if (yp.lt.y(1)) then
        lsp=1
        yp=y(1)
      endif 
      if (yp.gt.y(nr)) then
        lnp=1
        yp=y(nr)
      endif 

c need first to establish, xscale, yscale
c loop over longitude
      xscale=9999.
      yscale=9999.
      zscale=9999.
        do 50 i=1,nc
          ip1=i+1
          if(i.eq.nc)ip1=1
          xlon=x(i) 
          xlonp1=x(ip1)
          if(i.eq.nc)xlonp1=360.+x(ip1)
          if((xp.ge.xlon).and.(xp.le.xlonp1))then
c            print *,' x point found', xp,xlon,xlonp1
            xscale=(xp-xlon)/(xlonp1-xlon)
c loop over latitude
            do 40 j=1,nr-1
              jp1=j+1
              xlat=y(j)
              xlatp1=y(jp1)
              if((yp.ge.xlat).and.(yp.le.xlatp1))then
c                print *,' y point found', yp,xlat,xlatp1
                yscale=(yp-xlat)/(xlatp1-xlat)
ctdf Now have xscale, yscale for chosen parcel, n
ctdf Find surface pressure that chosen location
                psj=psfcgrd(j,i)+
     +              xscale*(psfcgrd(j,ip1)-psfcgrd(j,i))
                psjp=psfcgrd(jp1,i)+
     +              xscale*(psfcgrd(jp1,ip1)-psfcgrd(jp1,i))
                ps=psj+yscale*(psjp-psj)
c                print *,' xscale,yscale,ps', xscale,yscale,ps

ctdf Now construct pressure profile at location
                do l = 1, nlw
                  pw(l) = lpnmc(l)
                  zw(l) = lpnmc(l)
                end do
c                print *,' pw =',pw
ctdf Test for vertical domain
                if (pp .lt. pw(1)) pp = pw(1)
                if (pp .gt. pw(nlw)) pp = pw(nlw)
ctdf Pressure profile constructed at location
                do l = 1, nlw-1
                  lp1 = l+1
                  pl = pw(l)
                  plp1 = pw(lp1)
                  if((pp.ge.pl).and.(pp.le.plp1))then
c                    print *,' z point found', pp,pl,plp1
ctdf linear in p    zscale=(pp-pl)/(plp1-pl)
ctdf linear log(p)
                    zscale=alog(pp/pl)/alog(plp1/pl)
c
                    uj1=ugrd(j,i,l)
     +                 +xscale*(ugrd(j,ip1,l)-ugrd(j,i,l))
                    vj1=vgrd(j,i,l)
     +                 +xscale*(vgrd(j,ip1,l)-vgrd(j,i,l))
c vertical velocity 
                    wj1=wgrd(j,i,l)
     +                 +xscale*(wgrd(j,ip1,l)-wgrd(j,i,l))
c
                   ujp1=ugrd(jp1,i,l)
     +                 +xscale*(ugrd(jp1,ip1,l)-ugrd(jp1,i,l))
                   vjp1=vgrd(jp1,i,l)
     +                 +xscale*(vgrd(jp1,ip1,l)-vgrd(jp1,i,l))
c vertical velocity 
                   wjp1=wgrd(jp1,i,l)
     +                 +xscale*(wgrd(jp1,ip1,l)-wgrd(jp1,i,l))
c
c level lp1
                    uj2=ugrd(j,i,lp1)
     +                 +xscale*(ugrd(j,ip1,lp1)-ugrd(j,i,lp1))
                    vj2=vgrd(j,i,lp1)
     +                 +xscale*(vgrd(j,ip1,lp1)-vgrd(j,i,lp1))
c vertical velocity 
                    wj2=wgrd(j,i,lp1)
     +                 +xscale*(wgrd(j,ip1,lp1)-wgrd(j,i,lp1))
c
                   ujp2=ugrd(jp1,i,lp1)
     +                 +xscale*(ugrd(jp1,ip1,lp1)-ugrd(jp1,i,lp1))
                   vjp2=vgrd(jp1,i,lp1)
     +                 +xscale*(vgrd(jp1,ip1,lp1)-vgrd(jp1,i,lp1))
c vertical velocity 
                   wjp2=wgrd(jp1,i,lp1)
     +                 +xscale*(wgrd(jp1,ip1,lp1)-wgrd(jp1,i,lp1))
c
                     u1=uj1+yscale*(ujp1-uj1)
                     v1=vj1+yscale*(vjp1-vj1)
c vertical velocity 
                     w1=wj1+yscale*(wjp1-wj1)
c
                     u2=uj2+yscale*(ujp2-uj2)
                     v2=vj2+yscale*(vjp2-vj2)
c vertical velocity 
                     w2=wj2+yscale*(wjp2-wj2)
c
                      u=u1+zscale*(u2-u1)
                      v=v1+zscale*(v2-v1)
c vertical velocity 
                      w=w1+zscale*(w2-w1)
                      wmag=sqrt(u**2+v**2)
ctdf temporary print
c                      print *,' x,y,p,u,v,w',xp,yp,pp,u,v,w
c      print *,' i,ip1,j,jp1',i,ip1,j,jp1
c      print *,' xscale, yscale, zscale ',xscale,yscale,zscale
                      goto 50
                  endif
               end do
             endif
   40     continue
        endif
50    continue
ctdf
ctdf
      xp=xpold
      yp=ypold
      pp=ppold
c
c top/bottom points
c
      if ( (pp .eq. pw(1)) .or. (pp .eq. pw(nlw)) ) w=0.0
c
c pole points
c
      if((lsp.eq.1).or.(lnp.eq.1))u=0.
      return
      end  

      subroutine flight_path(theta)
c
c*****************************************************************c
c                                                                 c
c   Initialize along ER2 flight path or DIAL curtain              c
c                                                                 c
c*****************************************************************c
c
      include 'comm_gfs_3d_v01.'
      real lat1,lon1,nlat,nlon
      character*120 test
      PI2 = 6.2831853071796
      DTR = ASIN(1.0)/90. 
      RAD=6.37E6
      RADG = PI2 / 360.
      ds=2000.
      len=0
      nt=0
      nz=0
      n=1

      if(er2)then
      do leg=1,nleg                              
         olon=xleg0(leg)                        
         olat=yleg0(leg)              
         flon=xleg1(leg)                        
         flat=yleg1(leg)                        
         blat=.5*(olat+flat)          
         sx=pi2*rad*(flon-olon)*cos(flat*dtr)/360.
         sy=pi2*rad*(flat-olat)/360.
         s=sqrt(sx**2+sy**2)
         xn(n)=olon
	 yn(n)=olat
         zn(n)=theta
         iflag=0
         olon=xn(n)
         olat=yn(n)
         nstep=s/ds
         write(6,*)nstep,olat,olon,flat,flon,sx,sy,dx,dy
         if (nstep.gt.0) then 
             dx=(flon-olon)/nstep
             dy=(flat-olat)/nstep
             do 75 ISTEP=1,NSTEP
c
c  UPDATE TRAJECTORY
                xn(N+1)=olon+dx
                yn(N+1)=olat+dy
                zn(N+1)=theta

                IF (xn(N+1) .LT. 0.) xn(N+1)=360.+xn(N+1)
                IF (yn(N+1) .GT. 90.) THEN 
                    yn(N+1)=180.-yn(N+1)
                    xn(N+1)=180.+xn(N+1)
                ENDIF
                IF (yn(N+1) .LT. -90.) THEN 
                    yn(N+1)=-180.-yn(N+1)
                    xn(N+1)=180.+xn(N+1)
                ENDIF
                IF (xn(N+1) .GT. 360.) THEN 
                    MULT=xn(N+1)/360.
                    xn(N+1)=xn(N+1)-MULT*360.
                ENDIF
                n=n+1
                if(n.ge.nmax)return
	        olon=xn(n)
                olat=yn(n)
75           continue
         endif
         write(6,*)leg,n
      enddo
      endif

      ntraj=n
      return
      END
      subroutine igwave(uigw,vigw,u,v,i,j,l,xp,yp,zp,wmag,theta) 
c
c*****************************************************************c
c                                                                 c
c   Compute inertial gravity wave perturbation                    c
c                                                                 c
c*****************************************************************c
c
      include 'comm_gfs_3d_v01.'
      real m,l,k,n2,f,r,omega,horz,pi,radg
      PI2 = 6.2831853071796
      RAD=6.37E6
      radg=pi2/360.
      pi=.5*pi2
      t=gmt*3600.
c
c major axis of elliptically polarized IGW oriented in direction of jet max
c
      if(v/wmag.gt.1.)v=wmag
      if(v/wmag.lt.-1.)v=-wmag
      theta1=acos(v/wmag)
      if((v.gt.0.).and.(u.lt.0.)) 
     &  theta1=2.*pi-theta
      if((v.lt.0.).and.(u.lt.0.)) 
     &  theta1=theta+pi/2.
c
c major axis of elliptically polarized IGW oriented 45 degrees clockwise on
c cyclonic side of jet max and 45 degrees counter clockwise on anticyclonic 
c side of jet max (like a bow wave)
c
      jp1=j+1
      jm1=j-1
      ip1=i+1
      im1=i-1
      if(i.eq.nc)ip1=1
      if(i.eq.1)im1=nc
      if(j.eq.nr)jp1=j
      if(j.eq.1)jm1=1
      dudy=ugrd(jm1,i,l)-ugrd(jp1,i,l)
      dvdx=vgrd(j,ip1,l)-vgrd(j,im1,l)
c      goto 10
      if((u.ge.0.).and.(v.ge.0.)) then 
        if((dvdx.le.0.).and.(dudy.ge.0.)) theta1=theta1+pi/4. 
        if((dvdx.ge.0.).and.(dudy.le.0.)) theta1=theta1-pi/4. 
        goto 10
      endif
      if((u.ge.0.).and.(v.le.0.)) then 
        if((dvdx.le.0.).and.(dudy.ge.0.)) theta1=theta1+pi/4. 
        if((dvdx.ge.0.).and.(dudy.le.0.)) theta1=theta1-pi/4. 
        goto 10
      endif
      if((u.le.0.).and.(v.le.0.)) then 
        if((dvdx.le.0.).and.(dudy.ge.0.)) theta1=theta1+pi/4. 
        if((dvdx.ge.0.).and.(dudy.le.0.)) theta1=theta1-pi/4. 
        goto 10
      endif
      if((u.le.0.).and.(v.ge.0.)) then 
        if((dvdx.le.0.).and.(dudy.ge.0.)) theta1=theta1+pi/4. 
        if((dvdx.ge.0.).and.(dudy.le.0.)) theta1=theta1-pi/4. 
        goto 10
      endif
10    continue
c
c vertical wavelength (m-1) 
c
      m=1./3.e3                          
c
c meridional wavelength (m-1)
c
      l=1./100.e3                        
c
c longitudinal wavelength (m-1) 
c
      k=1./100.e3                        
c
c B-V frequency squared (s-2)
c
      n2=9.8*(log(1500.)-log(850.))/10e3  
c
c coriolis (s-1)
c
      f=2.*7.292e-5*sin(y(j)*radg)           
c
c gas constant 
c
      r=287.                             
c
c 6 hr inertial gravity wave frequency 
c
      omega=-1./(6.*3600.)
      horz=((2.*pi*omega)**2-f**2)*m**2/n2
      k=sin(theta1)*sqrt(horz)
      l=cos(theta1)*sqrt(horz)
c
c temperature perturbation
c                              
c temp0=3.*exp(-(wmax-wmag)**2/(wmax-70.)**2)
c
c temperature perturbation
c                              
c temp0=1.*exp(-(wmax-wmag)**2/(wmax-70.)**2)
c
c temperature perturbation
c                              
      temp0=3.*exp(-(wmax-wmag)**2/(wmax-70.)**2)
      dx=xp-x(i)
      dy=yp-y(j)
      if(dx.gt.360./96.) dx=dx-360.
      if(dx.lt.-360./96.) dx=dx+360.
      x1=rad*dx*radg*cos(y(j)*radg)
      y1=rad*dy*radg
c
c  inertial gravity wave
c
      temp=sin(2.*pi*(k*x1+l*y1-
     &           (omega+k*u+l*v)*t))+
     &     cos(2.*pi*(k*x1+l*y1-
     &           (omega+k*u+l*v)*t))
      uigw=k*omega*sin(2.*pi*(k*x1+l*y1-
     &                   (omega+k*u+l*v)*t))-
     &         l*f*cos(2.*pi*(k*x1+l*y1-
     &                   (omega+k*u+l*v)*t))
      vigw=l*omega*sin(2.*pi*(k*x1+l*y1-
     &                   (omega+k*u+l*v)*t))+
     &         k*f*cos(2.*pi*(k*x1+l*y1-
     &                   (omega+k*u+l*v)*t))
      temp=temp0*temp
      uigw=r*temp0*uigw/((2.*pi*omega)**2-f**2)
      vigw=r*temp0*vigw/((2.*pi*omega)**2-f**2)
      return
      end

      subroutine read_dial 
C
c*****************************************************************c
c                                                                 c
c   Read DIAL curtain data                                        c
c                                                                 c
c*****************************************************************c
c
      include 'comm_gfs_3d_v01.'
      character*120 test
      PI2 = 6.2831853071796
      DTR = ASIN(1.0)/90. 
      RAD=6.37E6
      RADG = PI2 / 360.
      ds=2000.
      len=0
      lend=0
      nt=0
      nz=0
      test=dfile(1)
      do nf=1,119
         if(test(nf:nf+1).ne.' ')len=len+1
         if(dird(nf:nf+1).ne.' ')lend=lend+1
      enddo
c We need the DIAL date for DIAL initialisation        Duncan 7/1/99 
c
      OPEN(99,FILE=dird(1:lend)//test(1:len),status='old',
     &     READONLY,FORM='FORMATTED')
      print *,'opening ',dird(1:lend)//test(1:len)
      read(99,*)dial_date 
      print *,' dial_date =',dial_date 
      read(99,*)nt
      print *,'        nt =',nt  
      read(99,*)(xsave(k),k=1,nt),(ysave(k),k=1,nt),
     &          (tsave(k),k=1,nt)
      print *,' xsave ',(xsave(k),k=1,6)
      print *,' ysave ',(ysave(k),k=1,6)
      print *,' tsave ',(tsave(k),k=1,6)
      read(99,*) nt, nz
      print *,'   nt, nz  =',nt, nz   
      read(99,*)((o3save(k,l),k=1,nt),l=1,nz),
     &          (( psave(k,l),k=1,nt),l=1,nz),
     &          ((thsave(k,l),k=1,nt),l=1,nz),
     &          ((pvsave(k,l),k=1,nt),l=1,nz),
     &          (( usave(k,l),k=1,nt),l=1,nz),
     &          (( vsave(k,l),k=1,nt),l=1,nz),
     &          (( qsave(k,l),k=1,nt),l=1,nz),
     &        ((xmsfsave(k,l),k=1,nt),l=1,nz),
     &         ((qdfsave(k,l),k=1,nt),l=1,nz)
ctdf     &     ((xmrksf_save(k,l),k=1,nt),l=1,nz)
      if (dt .ge. 0.0) then               ! forward trajectory condition  
        idial = 1                         ! address first profile 
        ncount = nt 
      endif 
      if (dt .lt. 0.0) then               ! back trajectory condition 
        idial = nt                        ! address final profile 
        ncount = nt 
      endif 
      print *,' DIAL file read OK, ncount =',ncount 
      return
      end 

      subroutine read_curtain
C
c*****************************************************************c
c                                                                 c
c   Read Curtain constituent and dynamical data                   c
c                                                                 c
c*****************************************************************c
c
      include 'comm_gfs_3d_v01.'
      character*120 test
      PI2 = 6.2831853071796
      DTR = ASIN(1.0)/90. 
      RAD=6.37E6
      RADG = PI2 / 360.
      ds=2000.
      len=0
      lend=0
      nt=0
      nz=0
      test=cfile(1)
      do nf=1,119
         if(test(nf:nf+1).ne.' ')len=len+1
         if(dirc(nf:nf+1).ne.' ')lend=lend+1
      enddo
c We need the Curtain date for initialisation        tdf 4/18/2K5 
c
      OPEN(99,FILE=dirc(1:lend)//test(1:len),status='old',
     &     READONLY,FORM='FORMATTED')
      print *,'opening ',dirc(1:lend)//test(1:len)
      read(99,*)dial_date 
      print *,' curtain_date =',dial_date 
      read(99,*)nt
      print *,'        nt =',nt  
      read(99,*)(xsave(k),k=1,nt),(ysave(k),k=1,nt),
     &          (tsave(k),k=1,nt)
      print *,' xsave ',(xsave(k),k=1,6)
      print *,' ysave ',(ysave(k),k=1,6)
      print *,' tsave ',(tsave(k),k=1,6)
      read(99,*) nt, nz
      print *,'   nt, nz  =',nt, nz   
      read(99,*)((pvsave(k,l),k=1,nt),l=1,nz),
     &          (( psave(k,l),k=1,nt),l=1,nz),
     &        ((xmsfsave(k,l),k=1,nt),l=1,nz),
     &          (( qsave(k,l),k=1,nt),l=1,nz),
     &          ((shsave(k,l),k=1,nt),l=1,nz),
     &          ((thsave(k,l),k=1,nt),l=1,nz),
     &          ((qdfsave(k,l),k=1,nt),l=1,nz)
      if (dt .ge. 0.0) then               ! forward trajectory condition  
        idial = 1                         ! address first profile 
        ncount = nt 
      endif 
      if (dt .lt. 0.0) then               ! back trajectory condition 
        idial = nt                        ! address final profile 
        ncount = nt 
      endif 
      print *,' Curtain file read OK, ncount =',ncount 
      return
      end 


      subroutine read_calipso_dat
C
c*****************************************************************c
c                                                                 c
c   Read Curtain constituent and dynamical data                   c
c   Temporary routine to read in the positional characteristics   c
c   of the CALIPSO data. Positional data lacks dynamical          c
c   characteristics.                                              c
c                                                                 c
c*****************************************************************c
c
      include 'comm_gfs_3d_v01.'
      character*120 test
      PI2 = 6.2831853071796
      DTR = ASIN(1.0)/90. 
      RAD=6.37E6
      RADG = PI2 / 360.
      ds=2000.
      len=0
      lend=0
      nt=0
      nz=0
      test=cfile(1)
      do nf=1,119
         if(test(nf:nf+1).ne.' ')len=len+1
         if(dirc(nf:nf+1).ne.' ')lend=lend+1
      enddo
c We need the Curtain date for initialisation        tdf 4/18/2K5 
c
      print *,' called read_calipso routine ' 
      OPEN(99,FILE=dirc(1:lend)//test(1:len),status='old',
     &     READONLY,FORM='unformatted')
      print *,'opening ',dirc(1:lend)//test(1:len)
      read(99)dial_date 
      print *,' calipso_date =',dial_date 
      call flush(6)
      read(99)nt
      print *,'        nt =',nt  
      call flush(6)
      read(99)(xsave(k),k=1,nt),(ysave(k),k=1,nt),
     &          (tsave(k),k=1,nt)
      print *,' xsave ',(xsave(k),k=1,6)
      print *,' ysave ',(ysave(k),k=1,6)
      print *,' tsave ',(tsave(k),k=1,6)
      call flush(6)
      read(99) nt, nz
      print *,'   nt, nz  =',nt, nz   
      call flush(6)
      read(99)((altsave(k,l),k=1,nt),l=1,nz),
     &          ((psave(k,l),k=1,nt),l=1,nz),
c following arrays may contain 99999.               tdf 8/22/2K5
     &          ((pvsave(k,l),k=1,nt),l=1,nz),
     &          ((thsave(k,l),k=1,nt),l=1,nz),
     &          ((shsave(k,l),k=1,nt),l=1,nz),
     &          ((qdfsave(k,l),k=1,nt),l=1,nz),
     &          ((xmsfsave(k,l),k=1,nt),l=1,nz)
      print *,' altsave ',(altsave(1,l),l=1,nz)
      print *,' shsave ',(shsave(1,l),l=1,nz)
      if (dt .ge. 0.0) then               ! forward trajectory condition  
        idial = 1                         ! address first profile 
        ncount = nt 
      endif 
      if (dt .lt. 0.0) then               ! back trajectory condition 
        idial = nt                        ! address final profile 
        ncount = nt 
      endif 
      print *,' Curtain file read OK, ncount =',ncount 
      return
      end 

      subroutine read_calipso
C
c*****************************************************************c
c                                                                 c
c   Read Curtain constituent and dynamical data                   c
c   Temporary routine to read in the positional characteristics   c
c   of the CALIPSO data. Positional data lacks dynamical          c
c   characteristics.                                              c
c                                                                 c
c*****************************************************************c
c
      include 'comm_gfs_3d_v01.'
      character*120 test
      PI2 = 6.2831853071796
      DTR = ASIN(1.0)/90. 
      RAD=6.37E6
      RADG = PI2 / 360.
      ds=2000.
      len=0
      lend=0
      nt=0
      nz=0
      test=cfile(1)
      do nf=1,119
         if(test(nf:nf+1).ne.' ')len=len+1
         if(dirc(nf:nf+1).ne.' ')lend=lend+1
      enddo
c We need the Curtain date for initialisation        tdf 4/18/2K5 
c
      OPEN(99,FILE=dirc(1:lend)//test(1:len),status='old',
     &     READONLY,FORM='FORMATTED')
      print *,'opening ',dirc(1:lend)//test(1:len)
      read(99,*)dial_date 
      print *,' calipso_date =',dial_date 
      call flush(6)
      read(99,*)nt
      print *,'        nt =',nt  
      call flush(6)
      read(99,*)(xsave(k),k=1,nt),(ysave(k),k=1,nt),
     &          (tsave(k),k=1,nt)
      print *,' xsave ',(xsave(k),k=1,6)
      print *,' ysave ',(ysave(k),k=1,6)
      print *,' tsave ',(tsave(k),k=1,6)
      call flush(6)
      read(99,*) nt, nz
      print *,'   nt, nz  =',nt, nz   
      call flush(6)
      read(99,*)((zsave(k,l),k=1,nt),l=1,nz),
     &          ((psave(k,l),k=1,nt),l=1,nz),
c following arrays may contain 99999.               tdf 8/22/2K5
     &          ((pvsave(k,l),k=1,nt),l=1,nz),
     &          ((thsave(k,l),k=1,nt),l=1,nz),
     &          ((shsave(k,l),k=1,nt),l=1,nz),
     &          ((qdfsave(k,l),k=1,nt),l=1,nz),
     &          ((xmsfsave(k,l),k=1,nt),l=1,nz)
      if (dt .ge. 0.0) then               ! forward trajectory condition  
        idial = 1                         ! address first profile 
        ncount = nt 
      endif 
      if (dt .lt. 0.0) then               ! back trajectory condition 
        idial = nt                        ! address final profile 
        ncount = nt 
      endif 
      print *,' Curtain file read OK, ncount =',ncount 
      return
      end 


      subroutine read_dc8_annulus
C
c*****************************************************************c
c                                                                 c
c   Read DC-8 annulus data                                        c
c                                                                 c
c*****************************************************************c
c
      include 'comm_gfs_3d_v01.'
      character*120 test
      PI2 = 6.2831853071796
      DTR = ASIN(1.0)/90.
      RAD=6.37E6
      RADG = PI2 / 360.
      ds=2000.
      len=0
      lend=0
      nt=0
      nz=0
      nh=0
      nv=0
      test=dc8file(1)
      do nf=1,119
         if(test(nf:nf+1).ne.' ')len=len+1
         if(dirdc8(nf:nf+1).ne.' ')lend=lend+1
      enddo
c We need the DC-8 date for DC-8 initialisation        Duncan 7/1/99
c
      OPEN(99,FILE=dirdc8(1:lend)//test(1:len),status='old',
     &     READONLY,FORM='FORMATTED')
      print *,'opening ',dirdc8(1:lend)//test(1:len)
      read(99,*)dial_date
      print *,' dial_date =',dial_date
      read(99,*)nt
      print *,'        nt =',nt
      read(99,*)(tsave(k),k=1,nt)
      read(99,*)nt,nh,nv
      print *,'  nt =',nt,'  nh =',nh,'  nv =',nv
c      read(99,*)
c     & ((( xsave3d(k,l,n),k=1,nt),l=1,nh),n=1,nv),
c     & ((( ysave3d(k,l,n),k=1,nt),l=1,nh),n=1,nv),
c     & ((( zsave3d(k,l,n),k=1,nt),l=1,nh),n=1,nv),
c     & ((( o3save3d(k,l,n),k=1,nt),l=1,nh),n=1,nv),
c     & (((  psave3d(k,l,n),k=1,nt),l=1,nh),n=1,nv),
c     & ((( thsave3d(k,l,n),k=1,nt),l=1,nh),n=1,nv),
c     & ((( pvsave3d(k,l,n),k=1,nt),l=1,nh),n=1,nv),
c     & (((  usave3d(k,l,n),k=1,nt),l=1,nh),n=1,nv),
c     & (((  vsave3d(k,l,n),k=1,nt),l=1,nh),n=1,nv),
c     & (((  qsave3d(k,l,n),k=1,nt),l=1,nh),n=1,nv),
c     & (((xmsfsave3d(k,l,n),k=1,nt),l=1,nh),n=1,nv),
c     & (((qdfsave3d(k,l,n),k=1,nt),l=1,nh),n=1,nv)
ctdf     & (((xmrksf_save3d(k,l,n),k=1,nt),l=1,nh),n=1,nv)

      if (dt .ge. 0.0) then               ! forward trajectory condition
        idial = 1                         ! address first profile
        ncount = nt
      endif
      if (dt .lt. 0.0) then               ! back trajectory condition
        idial = nt                        ! address final profile
        ncount = nt
      endif
      print *,' DC-8 file read OK, ncount =',ncount
      return
      end

ctdf      subroutine init_dial(idial)
      subroutine init_dial
c
c*****************************************************************c
c                                                                 c
c   Initialise DIAL data profile # idial                          c
c                                                                 c
c*****************************************************************c
c
      include 'comm_gfs_3d_v01.'
c
c initialize x,y,p coincident with time, altitude curtain
c
      ist = ntraj+1 
      n = ist 
      do k = 1, nz
C include next 2 lines to exclude missing data points 
          if((o3save(idial,k).lt.99999.) .and.
     1       (psave(idial,k).lt.99999.)) then
           xn(n)=xsave(idial)
           yn(n)=ysave(idial)
           zn(n)=thsave(idial,k)
           pn(n)=psave(idial,k)
           pvn(n)=pvsave(idial,k)
           xmsfn(n)=xmsfsave(idial,k)
           shn(n)=qsave(idial,k)   
ctdf           thdn(n)=qsave(idial,k)
           qdfn(n)=qdfsave(idial,k)
ctdf           xmrksfn(n)=xmrksf_save(idial,k)
c Adding t0n and agen arrays                       Duncan 11/05/99
           t0n(n)=tsave(idial)
           agen(n)=0.0
           n = n+1 
          endif
      enddo
      ntraj = n-1
C
      return
      end

ctdf      subroutine init_curtain(idial)
      subroutine init_curtain
c
c*****************************************************************c
c                                                                 c
c   Initialise Curtain data profile # idial                       c
c                                                                 c
c*****************************************************************c
c
      include 'comm_gfs_3d_v01.'
c
c initialize x,y,p coincident with time, altitude curtain
c
      ist = ntraj+1 
      n = ist 
ctdf Loop over model eta levels              tdf 4/18/2K5
      do k = 1, nthwmx
C include next 2 lines to exclude missing data points 
          if((o3save(idial,k).lt.99999.) .and.
     1       (psave(idial,k).lt.99999.)) then
           xn(n)=xsave(idial)
           yn(n)=ysave(idial)
           zn(n)=thw(k)                      ! model eta levels
           thn(n)=thsave(idial,k)            ! theta
           pn(n)=psave(idial,k)
           pvn(n)=pvsave(idial,k)
           xmsfn(n)=xmsfsave(idial,k)
           shn(n)=qsave(idial,k)   
ctdf           thdn(n)=qsave(idial,k)
           qdfn(n)=qdfsave(idial,k)
           shn(n)=shsave(idial,k)
ctdf           xmrksfn(n)=xmrksf_save(idial,k)
c Adding t0n and agen arrays                       Duncan 11/05/99
           t0n(n)=tsave(idial)
           agen(n)=0.0
           n = n+1 
          endif
      enddo
      ntraj = n-1
C
      return
      end


      subroutine init_calipso(ist,last)
c
c*****************************************************************c
c                                                                 c
c   Initialise calipso data profile # idial                       c
c                                                                 c
c*****************************************************************c
c
      include 'comm_gfs_3d_v01.'
c
c initialize x,y,p coincident with time, altitude curtain
c
      ist = ntraj+1 
      n = ist 
ctdf Loop over model eta levels              tdf 4/18/2K5
      do k = 1, nz
C include next 2 lines to exclude missing data points 
         if (psave(idial,k).lt.99999.) then
           xn(n)=xsave(idial)
           yn(n)=ysave(idial)
           altn(n)=altsave(idial,k)          ! geop. altitude
           zn(n)=zsave(idial,k)              ! model eta levels ?
           thn(n)=thsave(idial,k)            ! theta
           pn(n)=psave(idial,k)
           pvn(n)=pvsave(idial,k)
           xmsfn(n)=xmsfsave(idial,k)
ctdf           thdn(n)=qsave(idial,k)
           qdfn(n)=qdfsave(idial,k)
           shn(n)=shsave(idial,k)
ctdf           xmrksfn(n)=xmrksf_save(idial,k)
c Adding t0n and agen arrays                       Duncan 11/05/99
           t0n(n)=tsave(idial)
           agen(n)=0.0
           n = n+1 
          endif
      enddo
      ntraj = n-1
      last = n-1
C
      return
      end


ctdf      subroutine init_dc8(idial)
      subroutine init_dc8
c
c*****************************************************************c
c                                                                 c
c   Initialise DC-8 20x20 annulus at profile # idial              c
c                                                                 c
c*****************************************************************c
c
      include 'comm_gfs_3d_v01.'
c
c initialize x,y,p coincident with time, annulus array
c
      ist = ntraj+1
      n = ist
c      do j = 1, nhbin
c      do k = 1, nvbin
c           xn(n)=xsave3d(idial,j,k)
c           yn(n)=ysave3d(idial,j,k)
c           thn(n)=thsave3d(idial,j,k)
c           pn(n)=psave3d(idial,j,k)
c           pvn(n)=pvsave3d(idial,j,k)
c           xmsfn(n)=xmsfsave3d(idial,j,k)
c           shn(n)=qsave3d(idial,j,k)
cctdf           thdn(n)=qsave3d(idial,j,k)
c           qdfn(n)=qdfsave3d(idial,j,k)
cctdf           xmrksfn(n)=xmrksf_save3d(idial,j,k)
cc Adding t0n and agen arrays                       Duncan 11/05/99
c           t0n(n)=0.0
c           agen(n)=0.0
c           n = n+1
c           if(n.ge.nmax)stop 'n exceeds nmax'
c      enddo
c      enddo
      ntraj = n-1

      return
      end

      subroutine zero_arrays
      include 'comm_gfs_3d_v01.'
      do j = 1, nmax
c positional
        xn(j)=99999.
        yn(j)=99999.
        zn(j)=99999.
c dynamical/diagnostic
        pvn(j)=99999.
        pn(j)=99999.
ctdf potential temperature                    tdf 11/05/2K4
        thn(j)=99999.
        xmsfn(j)=99999.
        un(j)=99999.
        vn(j)=99999.
        shn(j)=99999.
ctdf        thdn(j)=99999.
        qdfn(j)=99999.
ctdf        xmrksfn(j)=99999.
ctdf specific humidity
        shn(j)=99999.
c Temporal quantities 
        agen(j)=99999.
        t0n(j)=99999.
c daylight summed quantities
ctdf        frday(j)=0.0
ctdf        frnght(j)=0.0
c minimum temperature 
ctdf        xmint(j)=99999.
      end do
      return
      end

